from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from enum import Enum

from typing import List

from box_sdk_gen.schemas.file_base import FileBaseTypeField

from box_sdk_gen.schemas.file_base import FileBase

from box_sdk_gen.schemas.file_version_mini import FileVersionMini

from box_sdk_gen.schemas.file_mini import FileMini

from box_sdk_gen.schemas.file import FilePathCollectionField

from box_sdk_gen.schemas.file import FileSharedLinkField

from box_sdk_gen.schemas.folder_mini import FolderMini

from box_sdk_gen.schemas.file import FileItemStatusField

from box_sdk_gen.schemas.file import File

from box_sdk_gen.schemas.user_mini import UserMini

from box_sdk_gen.schemas.resource_scope import ResourceScope

from box_sdk_gen.schemas.metadata_full import MetadataFull

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class FileFullPermissionsField(BaseObject):
    def __init__(
        self,
        can_delete: bool,
        can_download: bool,
        can_invite_collaborator: bool,
        can_rename: bool,
        can_set_share_access: bool,
        can_share: bool,
        can_annotate: bool,
        can_comment: bool,
        can_preview: bool,
        can_upload: bool,
        can_view_annotations_all: bool,
        can_view_annotations_self: bool,
        *,
        can_apply_watermark: Optional[bool] = None,
        **kwargs
    ):
        """
                :param can_delete: Specifies if the current user can delete this item.
                :type can_delete: bool
                :param can_download: Specifies if the current user can download this item.
                :type can_download: bool
                :param can_invite_collaborator: Specifies if the current user can invite new
        users to collaborate on this item, and if the user can
        update the role of a user already collaborated on this
        item.
                :type can_invite_collaborator: bool
                :param can_rename: Specifies if the user can rename this item.
                :type can_rename: bool
                :param can_set_share_access: Specifies if the user can change the access level of an
        existing shared link on this item.
                :type can_set_share_access: bool
                :param can_share: Specifies if the user can create a shared link for this item.
                :type can_share: bool
                :param can_annotate: Specifies if the user can place annotations on this file.
                :type can_annotate: bool
                :param can_comment: Specifies if the user can place comments on this file.
                :type can_comment: bool
                :param can_preview: Specifies if the user can preview this file.
                :type can_preview: bool
                :param can_upload: Specifies if the user can upload a new version of this file.
                :type can_upload: bool
                :param can_view_annotations_all: Specifies if the user view all annotations placed on this file.
                :type can_view_annotations_all: bool
                :param can_view_annotations_self: Specifies if the user view annotations placed by themselves
        on this file.
                :type can_view_annotations_self: bool
                :param can_apply_watermark: Specifies if the user can apply a watermark to this file., defaults to None
                :type can_apply_watermark: Optional[bool], optional
        """
        super().__init__(**kwargs)
        self.can_delete = can_delete
        self.can_download = can_download
        self.can_invite_collaborator = can_invite_collaborator
        self.can_rename = can_rename
        self.can_set_share_access = can_set_share_access
        self.can_share = can_share
        self.can_annotate = can_annotate
        self.can_comment = can_comment
        self.can_preview = can_preview
        self.can_upload = can_upload
        self.can_view_annotations_all = can_view_annotations_all
        self.can_view_annotations_self = can_view_annotations_self
        self.can_apply_watermark = can_apply_watermark


class FileFullLockTypeField(str, Enum):
    LOCK = 'lock'


class FileFullLockAppTypeField(str, Enum):
    GSUITE = 'gsuite'
    OFFICE_WOPI = 'office_wopi'
    OFFICE_WOPIPLUS = 'office_wopiplus'
    OTHER = 'other'


class FileFullLockField(BaseObject):
    _discriminator = 'type', {'lock'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[FileFullLockTypeField] = None,
        created_by: Optional[UserMini] = None,
        created_at: Optional[DateTime] = None,
        expired_at: Optional[DateTime] = None,
        is_download_prevented: Optional[bool] = None,
        app_type: Optional[FileFullLockAppTypeField] = None,
        **kwargs
    ):
        """
                :param id: The unique identifier for this lock., defaults to None
                :type id: Optional[str], optional
                :param type: The value will always be `lock`., defaults to None
                :type type: Optional[FileFullLockTypeField], optional
                :param created_at: The time this lock was created at., defaults to None
                :type created_at: Optional[DateTime], optional
                :param expired_at: The time this lock is to expire at, which might be in the past., defaults to None
                :type expired_at: Optional[DateTime], optional
                :param is_download_prevented: Whether or not the file can be downloaded while locked., defaults to None
                :type is_download_prevented: Optional[bool], optional
                :param app_type: If the lock is managed by an application rather than a user, this
        field identifies the type of the application that holds the lock.
        This is an open enum and may be extended with additional values in
        the future., defaults to None
                :type app_type: Optional[FileFullLockAppTypeField], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.created_by = created_by
        self.created_at = created_at
        self.expired_at = expired_at
        self.is_download_prevented = is_download_prevented
        self.app_type = app_type


class FileFullExpiringEmbedLinkTokenTypeField(str, Enum):
    BEARER = 'bearer'


class FileFullExpiringEmbedLinkField(BaseObject):
    def __init__(
        self,
        *,
        access_token: Optional[str] = None,
        expires_in: Optional[int] = None,
        token_type: Optional[FileFullExpiringEmbedLinkTokenTypeField] = None,
        restricted_to: Optional[List[ResourceScope]] = None,
        url: Optional[str] = None,
        **kwargs
    ):
        """
                :param access_token: The requested access token., defaults to None
                :type access_token: Optional[str], optional
                :param expires_in: The time in seconds by which this token will expire., defaults to None
                :type expires_in: Optional[int], optional
                :param token_type: The type of access token returned., defaults to None
                :type token_type: Optional[FileFullExpiringEmbedLinkTokenTypeField], optional
                :param restricted_to: The permissions that this access token permits,
        providing a list of resources (files, folders, etc)
        and the scopes permitted for each of those resources., defaults to None
                :type restricted_to: Optional[List[ResourceScope]], optional
                :param url: The actual expiring embed URL for this file, constructed
        from the file ID and access tokens specified in this object., defaults to None
                :type url: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.access_token = access_token
        self.expires_in = expires_in
        self.token_type = token_type
        self.restricted_to = restricted_to
        self.url = url


class FileFullWatermarkInfoField(BaseObject):
    def __init__(
        self,
        *,
        is_watermarked: Optional[bool] = None,
        is_watermark_inherited: Optional[bool] = None,
        is_watermarked_by_access_policy: Optional[bool] = None,
        **kwargs
    ):
        """
        :param is_watermarked: Specifies if this item has a watermark applied., defaults to None
        :type is_watermarked: Optional[bool], optional
        :param is_watermark_inherited: Specifies if the watermark is inherited from any parent folder in the hierarchy., defaults to None
        :type is_watermark_inherited: Optional[bool], optional
        :param is_watermarked_by_access_policy: Specifies if the watermark is enforced by an access policy., defaults to None
        :type is_watermarked_by_access_policy: Optional[bool], optional
        """
        super().__init__(**kwargs)
        self.is_watermarked = is_watermarked
        self.is_watermark_inherited = is_watermark_inherited
        self.is_watermarked_by_access_policy = is_watermarked_by_access_policy


class FileFullAllowedInviteeRolesField(str, Enum):
    EDITOR = 'editor'
    VIEWER = 'viewer'
    PREVIEWER = 'previewer'
    UPLOADER = 'uploader'
    PREVIEWER_UPLOADER = 'previewer uploader'
    VIEWER_UPLOADER = 'viewer uploader'
    CO_OWNER = 'co-owner'


class FileFullMetadataField(BaseObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.extra_data = kwargs


class FileFullRepresentationsEntriesContentField(BaseObject):
    def __init__(self, *, url_template: Optional[str] = None, **kwargs):
        """
                :param url_template: The download URL that can be used to fetch the representation.
        Make sure to make an authenticated API call to this endpoint.

        This URL is a template and will require the `{+asset_path}` to
        be replaced by a path. In general, for unpaged representations
        it can be replaced by an empty string.

        For paged representations, replace the `{+asset_path}` with the
        page to request plus the extension for the file, for example
        `1.pdf`.

        When requesting the download URL the following additional
        query params can be passed along.

        * `set_content_disposition_type` - Sets the
        `Content-Disposition` header in the API response with the
        specified disposition type of either `inline` or `attachment`.
        If not supplied, the `Content-Disposition` header is not
        included in the response.

        * `set_content_disposition_filename` - Allows the application to
          define the representation's file name used in the
          `Content-Disposition` header.  If not defined, the filename
          is derived from the source file name in Box combined with the
          extension of the representation., defaults to None
                :type url_template: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.url_template = url_template


class FileFullRepresentationsEntriesInfoField(BaseObject):
    def __init__(self, *, url: Optional[str] = None, **kwargs):
        """
                :param url: The API URL that can be used to get more info on this file
        representation. Make sure to make an authenticated API call
        to this endpoint., defaults to None
                :type url: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.url = url


class FileFullRepresentationsEntriesPropertiesField(BaseObject):
    def __init__(
        self,
        *,
        dimensions: Optional[str] = None,
        paged: Optional[str] = None,
        thumb: Optional[str] = None,
        **kwargs
    ):
        """
                :param dimensions: The width by height size of this representation in pixels., defaults to None
                :type dimensions: Optional[str], optional
                :param paged: Indicates if the representation is build up out of multiple
        pages., defaults to None
                :type paged: Optional[str], optional
                :param thumb: Indicates if the representation can be used as a thumbnail of
        the file., defaults to None
                :type thumb: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.dimensions = dimensions
        self.paged = paged
        self.thumb = thumb


class FileFullRepresentationsEntriesStatusStateField(str, Enum):
    SUCCESS = 'success'
    VIEWABLE = 'viewable'
    PENDING = 'pending'
    NONE = 'none'


class FileFullRepresentationsEntriesStatusField(BaseObject):
    def __init__(
        self,
        *,
        state: Optional[FileFullRepresentationsEntriesStatusStateField] = None,
        **kwargs
    ):
        """
                :param state: The status of the representation.

        * `success` defines the representation as ready to be viewed.
        * `viewable` defines a video to be ready for viewing.
        * `pending` defines the representation as to be generated. Retry
          this endpoint to re-check the status.
        * `none` defines that the representation will be created when
          requested. Request the URL defined in the `info` object to
          trigger this generation., defaults to None
                :type state: Optional[FileFullRepresentationsEntriesStatusStateField], optional
        """
        super().__init__(**kwargs)
        self.state = state


class FileFullRepresentationsEntriesField(BaseObject):
    def __init__(
        self,
        *,
        content: Optional[FileFullRepresentationsEntriesContentField] = None,
        info: Optional[FileFullRepresentationsEntriesInfoField] = None,
        properties: Optional[FileFullRepresentationsEntriesPropertiesField] = None,
        representation: Optional[str] = None,
        status: Optional[FileFullRepresentationsEntriesStatusField] = None,
        **kwargs
    ):
        """
                :param content: An object containing the URL that can be used to actually fetch
        the representation., defaults to None
                :type content: Optional[FileFullRepresentationsEntriesContentField], optional
                :param info: An object containing the URL that can be used to fetch more info
        on this representation., defaults to None
                :type info: Optional[FileFullRepresentationsEntriesInfoField], optional
                :param properties: An object containing the size and type of this presentation., defaults to None
                :type properties: Optional[FileFullRepresentationsEntriesPropertiesField], optional
                :param representation: Indicates the file type of the returned representation., defaults to None
                :type representation: Optional[str], optional
                :param status: An object containing the status of this representation., defaults to None
                :type status: Optional[FileFullRepresentationsEntriesStatusField], optional
        """
        super().__init__(**kwargs)
        self.content = content
        self.info = info
        self.properties = properties
        self.representation = representation
        self.status = status


class FileFullRepresentationsField(BaseObject):
    def __init__(
        self,
        *,
        entries: Optional[List[FileFullRepresentationsEntriesField]] = None,
        **kwargs
    ):
        """
        :param entries: A list of files., defaults to None
        :type entries: Optional[List[FileFullRepresentationsEntriesField]], optional
        """
        super().__init__(**kwargs)
        self.entries = entries


class FileFullClassificationField(BaseObject):
    def __init__(
        self,
        *,
        name: Optional[str] = None,
        definition: Optional[str] = None,
        color: Optional[str] = None,
        **kwargs
    ):
        """
                :param name: The name of the classification., defaults to None
                :type name: Optional[str], optional
                :param definition: An explanation of the meaning of this classification., defaults to None
                :type definition: Optional[str], optional
                :param color: The color that is used to display the
        classification label in a user-interface. Colors are defined by the admin
        or co-admin who created the classification in the Box web app., defaults to None
                :type color: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.name = name
        self.definition = definition
        self.color = color


class FileFullSharedLinkPermissionOptionsField(str, Enum):
    CAN_PREVIEW = 'can_preview'
    CAN_DOWNLOAD = 'can_download'
    CAN_EDIT = 'can_edit'


class FileFull(File):
    def __init__(
        self,
        id: str,
        *,
        version_number: Optional[str] = None,
        comment_count: Optional[int] = None,
        permissions: Optional[FileFullPermissionsField] = None,
        tags: Optional[List[str]] = None,
        lock: Optional[FileFullLockField] = None,
        extension: Optional[str] = None,
        is_package: Optional[bool] = None,
        expiring_embed_link: Optional[FileFullExpiringEmbedLinkField] = None,
        watermark_info: Optional[FileFullWatermarkInfoField] = None,
        is_accessible_via_shared_link: Optional[bool] = None,
        allowed_invitee_roles: Optional[List[FileFullAllowedInviteeRolesField]] = None,
        is_externally_owned: Optional[bool] = None,
        has_collaborations: Optional[bool] = None,
        metadata: Optional[FileFullMetadataField] = None,
        expires_at: Optional[DateTime] = None,
        representations: Optional[FileFullRepresentationsField] = None,
        classification: Optional[FileFullClassificationField] = None,
        uploader_display_name: Optional[str] = None,
        disposition_at: Optional[DateTime] = None,
        shared_link_permission_options: Optional[
            List[FileFullSharedLinkPermissionOptionsField]
        ] = None,
        is_associated_with_app_item: Optional[bool] = None,
        description: Optional[str] = None,
        size: Optional[int] = None,
        path_collection: Optional[FilePathCollectionField] = None,
        created_at: Optional[DateTime] = None,
        modified_at: Optional[DateTime] = None,
        trashed_at: Optional[DateTime] = None,
        purged_at: Optional[DateTime] = None,
        content_created_at: Optional[DateTime] = None,
        content_modified_at: Optional[DateTime] = None,
        created_by: Optional[UserMini] = None,
        modified_by: Optional[UserMini] = None,
        owned_by: Optional[UserMini] = None,
        shared_link: Optional[FileSharedLinkField] = None,
        parent: Optional[FolderMini] = None,
        item_status: Optional[FileItemStatusField] = None,
        sequence_id: Optional[str] = None,
        name: Optional[str] = None,
        sha_1: Optional[str] = None,
        file_version: Optional[FileVersionMini] = None,
        etag: Optional[str] = None,
        type: FileBaseTypeField = FileBaseTypeField.FILE,
        **kwargs
    ):
        """
                :param id: The unique identifier that represent a file.

        The ID for any file can be determined
        by visiting a file in the web application
        and copying the ID from the URL. For example,
        for the URL `https://*.app.box.com/files/123`
        the `file_id` is `123`.
                :type id: str
                :param version_number: The version number of this file., defaults to None
                :type version_number: Optional[str], optional
                :param comment_count: The number of comments on this file., defaults to None
                :type comment_count: Optional[int], optional
                :param extension: Indicates the (optional) file extension for this file. By default,
        this is set to an empty string., defaults to None
                :type extension: Optional[str], optional
                :param is_package: Indicates if the file is a package. Packages are commonly used
        by Mac Applications and can include iWork files., defaults to None
                :type is_package: Optional[bool], optional
                :param is_accessible_via_shared_link: Specifies if the file can be accessed
        via the direct shared link or a shared link
        to a parent folder., defaults to None
                :type is_accessible_via_shared_link: Optional[bool], optional
                :param allowed_invitee_roles: A list of the types of roles that user can be invited at
        when sharing this file., defaults to None
                :type allowed_invitee_roles: Optional[List[FileFullAllowedInviteeRolesField]], optional
                :param is_externally_owned: Specifies if this file is owned by a user outside of the
        authenticated enterprise., defaults to None
                :type is_externally_owned: Optional[bool], optional
                :param has_collaborations: Specifies if this file has any other collaborators., defaults to None
                :type has_collaborations: Optional[bool], optional
                :param expires_at: When the file will automatically be deleted., defaults to None
                :type expires_at: Optional[DateTime], optional
                :param disposition_at: The retention expiration timestamp for the given file., defaults to None
                :type disposition_at: Optional[DateTime], optional
                :param shared_link_permission_options: A list of the types of roles that user can be invited at
        when sharing this file., defaults to None
                :type shared_link_permission_options: Optional[List[FileFullSharedLinkPermissionOptionsField]], optional
                :param is_associated_with_app_item: This field will return true if the file or any ancestor of the file
        is associated with at least one app item. Note that this will return
        true even if the context user does not have access to the app item(s)
        associated with the file., defaults to None
                :type is_associated_with_app_item: Optional[bool], optional
                :param description: The optional description of this file.
        If the description exceeds 255 characters, the first 255 characters
        are set as a file description and the rest of it is ignored., defaults to None
                :type description: Optional[str], optional
                :param size: The file size in bytes. Be careful parsing this integer as it can
        get very large and cause an integer overflow., defaults to None
                :type size: Optional[int], optional
                :param created_at: The date and time when the file was created on Box., defaults to None
                :type created_at: Optional[DateTime], optional
                :param modified_at: The date and time when the file was last updated on Box., defaults to None
                :type modified_at: Optional[DateTime], optional
                :param trashed_at: The time at which this file was put in the trash., defaults to None
                :type trashed_at: Optional[DateTime], optional
                :param purged_at: The time at which this file is expected to be purged
        from the trash., defaults to None
                :type purged_at: Optional[DateTime], optional
                :param content_created_at: The date and time at which this file was originally
        created, which might be before it was uploaded to Box., defaults to None
                :type content_created_at: Optional[DateTime], optional
                :param content_modified_at: The date and time at which this file was last updated,
        which might be before it was uploaded to Box., defaults to None
                :type content_modified_at: Optional[DateTime], optional
                :param item_status: Defines if this item has been deleted or not.

        * `active` when the item has is not in the trash
        * `trashed` when the item has been moved to the trash but not deleted
        * `deleted` when the item has been permanently deleted., defaults to None
                :type item_status: Optional[FileItemStatusField], optional
                :param name: The name of the file., defaults to None
                :type name: Optional[str], optional
                :param sha_1: The SHA1 hash of the file. This can be used to compare the contents
        of a file on Box with a local file., defaults to None
                :type sha_1: Optional[str], optional
                :param etag: The HTTP `etag` of this file. This can be used within some API
        endpoints in the `If-Match` and `If-None-Match` headers to only
        perform changes on the file if (no) changes have happened., defaults to None
                :type etag: Optional[str], optional
                :param type: The value will always be `file`., defaults to FileBaseTypeField.FILE
                :type type: FileBaseTypeField, optional
        """
        super().__init__(
            id=id,
            description=description,
            size=size,
            path_collection=path_collection,
            created_at=created_at,
            modified_at=modified_at,
            trashed_at=trashed_at,
            purged_at=purged_at,
            content_created_at=content_created_at,
            content_modified_at=content_modified_at,
            created_by=created_by,
            modified_by=modified_by,
            owned_by=owned_by,
            shared_link=shared_link,
            parent=parent,
            item_status=item_status,
            sequence_id=sequence_id,
            name=name,
            sha_1=sha_1,
            file_version=file_version,
            etag=etag,
            type=type,
            **kwargs
        )
        self.version_number = version_number
        self.comment_count = comment_count
        self.permissions = permissions
        self.tags = tags
        self.lock = lock
        self.extension = extension
        self.is_package = is_package
        self.expiring_embed_link = expiring_embed_link
        self.watermark_info = watermark_info
        self.is_accessible_via_shared_link = is_accessible_via_shared_link
        self.allowed_invitee_roles = allowed_invitee_roles
        self.is_externally_owned = is_externally_owned
        self.has_collaborations = has_collaborations
        self.metadata = metadata
        self.expires_at = expires_at
        self.representations = representations
        self.classification = classification
        self.uploader_display_name = uploader_display_name
        self.disposition_at = disposition_at
        self.shared_link_permission_options = shared_link_permission_options
        self.is_associated_with_app_item = is_associated_with_app_item



from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.v2025_r0.archives_v2025_r0 import ArchivesV2025R0

from box_sdk_gen.schemas.v2025_r0.client_error_v2025_r0 import ClientErrorV2025R0

from box_sdk_gen.parameters.v2025_r0.box_version_header_v2025_r0 import (
    BoxVersionHeaderV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.archive_v2025_r0 import ArchiveV2025R0

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class ArchivesManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_archives_v2025_r0(
        self,
        *,
        limit: Optional[int] = None,
        marker: Optional[str] = None,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ArchivesV2025R0:
        """
                Retrieves archives for an enterprise.

                To learn more about the archive APIs, see the [Archive API Guide](https://developer.box.com/guides/archives).

                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination., defaults to None
                :type marker: Optional[str], optional
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {'limit': to_string(limit), 'marker': to_string(marker)}
        )
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/archives']),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ArchivesV2025R0)

    def create_archive_v2025_r0(
        self,
        name: str,
        *,
        description: Optional[str] = None,
        storage_policy_id: Optional[str] = None,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ArchiveV2025R0:
        """
        Creates an archive.

        To learn more about the archive APIs, see the [Archive API Guide](https://developer.box.com/guides/archives).

        :param name: The name of the archive.
        :type name: str
        :param description: The description of the archive., defaults to None
        :type description: Optional[str], optional
        :param storage_policy_id: The ID of the storage policy that the archive is assigned to., defaults to None
        :type storage_policy_id: Optional[str], optional
        :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
        :type box_version: BoxVersionHeaderV2025R0, optional
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'description': description,
            'storage_policy_id': storage_policy_id,
        }
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/archives']),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ArchiveV2025R0)

    def delete_archive_by_id_v2025_r0(
        self,
        archive_id: str,
        *,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Permanently deletes an archive.

                To learn more about the archive APIs, see the [Archive API Guide](https://developer.box.com/guides/archives).


                <Danger>


                This endpoint is currently unavailable. Please contact support for assistance.


                </Danger>

                :param archive_id: The ID of the archive.
        Example: "982312"
                :type archive_id: str
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/archives/',
                        to_string(archive_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def update_archive_by_id_v2025_r0(
        self,
        archive_id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        box_version: BoxVersionHeaderV2025R0 = BoxVersionHeaderV2025R0._2025_0,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ArchiveV2025R0:
        """
                Updates an archive.

                To learn more about the archive APIs, see the [Archive API Guide](https://developer.box.com/guides/archives).

                :param archive_id: The ID of the archive.
        Example: "982312"
                :type archive_id: str
                :param name: The name of the archive., defaults to None
                :type name: Optional[str], optional
                :param description: The description of the archive., defaults to None
                :type description: Optional[str], optional
                :param box_version: Version header., defaults to BoxVersionHeaderV2025R0._2025_0
                :type box_version: BoxVersionHeaderV2025R0, optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'name': name, 'description': description}
        headers_map: Dict[str, str] = prepare_params(
            {'box-version': to_string(box_version), **extra_headers}
        )
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/archives/',
                        to_string(archive_id),
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ArchiveV2025R0)



from enum import Enum

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class WeblinkReferenceV2025R0TypeField(str, Enum):
    WEB_LINK = 'web_link'


class WeblinkReferenceV2025R0(BaseObject):
    _discriminator = 'type', {'web_link'}

    def __init__(
        self,
        id: str,
        *,
        type: WeblinkReferenceV2025R0TypeField = WeblinkReferenceV2025R0TypeField.WEB_LINK,
        **kwargs
    ):
        """
        :param id: ID of the web link.
        :type id: str
        :param type: The value will always be `web_link`., defaults to WeblinkReferenceV2025R0TypeField.WEB_LINK
        :type type: WeblinkReferenceV2025R0TypeField, optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type



from typing import Optional

from typing import List

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.task import Task

from box_sdk_gen.box.errors import BoxSDKError


class Tasks(BaseObject):
    def __init__(
        self,
        *,
        total_count: Optional[int] = None,
        entries: Optional[List[Task]] = None,
        **kwargs
    ):
        """
                :param total_count: One greater than the offset of the last entry in the entire collection.
        The total number of entries in the collection may be less than
        `total_count`., defaults to None
                :type total_count: Optional[int], optional
                :param entries: A list of tasks., defaults to None
                :type entries: Optional[List[Task]], optional
        """
        super().__init__(**kwargs)
        self.total_count = total_count
        self.entries = entries



from box_sdk_gen.internal.base_object import BaseObject

from typing import Optional

from typing import List

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.invite import Invite

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class CreateInviteEnterprise(BaseObject):
    def __init__(self, id: str, **kwargs):
        """
        :param id: The ID of the enterprise.
        :type id: str
        """
        super().__init__(**kwargs)
        self.id = id


class CreateInviteActionableBy(BaseObject):
    def __init__(self, *, login: Optional[str] = None, **kwargs):
        """
        :param login: The login of the invited user., defaults to None
        :type login: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.login = login


class InvitesManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def create_invite(
        self,
        enterprise: CreateInviteEnterprise,
        actionable_by: CreateInviteActionableBy,
        *,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> Invite:
        """
                Invites an existing external user to join an enterprise.

                The existing user can not be part of another enterprise and


                must already have a Box account. Once invited, the user will receive an


                email and are prompted to accept the invitation within the


                Box web application.


                This method requires the "Manage An Enterprise" scope enabled for


                the application, which can be enabled within the developer console.

                :param enterprise: The enterprise to invite the user to.
                :type enterprise: CreateInviteEnterprise
                :param actionable_by: The user to invite.
                :type actionable_by: CreateInviteActionableBy
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'enterprise': enterprise, 'actionable_by': actionable_by}
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/invites']),
                method='POST',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, Invite)

    def get_invite_by_id(
        self,
        invite_id: str,
        *,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> Invite:
        """
                Returns the status of a user invite.
                :param invite_id: The ID of an invite.
        Example: "213723"
                :type invite_id: str
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/invites/',
                        to_string(invite_id),
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, Invite)



from enum import Enum

from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.file_version_retentions import FileVersionRetentions

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.file_version_retention import FileVersionRetention

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class GetFileVersionRetentionsDispositionAction(str, Enum):
    PERMANENTLY_DELETE = 'permanently_delete'
    REMOVE_RETENTION = 'remove_retention'


class FileVersionRetentionsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_file_version_retentions(
        self,
        *,
        file_id: Optional[str] = None,
        file_version_id: Optional[str] = None,
        policy_id: Optional[str] = None,
        disposition_action: Optional[GetFileVersionRetentionsDispositionAction] = None,
        disposition_before: Optional[str] = None,
        disposition_after: Optional[str] = None,
        limit: Optional[int] = None,
        marker: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FileVersionRetentions:
        """
                Retrieves all file version retentions for the given enterprise.

                **Note**:


                File retention API is now **deprecated**.


                To get information about files and file versions under retention,


                see [files under retention](https://developer.box.com/reference/get-retention-policy-assignments-id-files-under-retention) or [file versions under retention](https://developer.box.com/reference/get-retention-policy-assignments-id-file-versions-under-retention) endpoints.

                :param file_id: Filters results by files with this ID., defaults to None
                :type file_id: Optional[str], optional
                :param file_version_id: Filters results by file versions with this ID., defaults to None
                :type file_version_id: Optional[str], optional
                :param policy_id: Filters results by the retention policy with this ID., defaults to None
                :type policy_id: Optional[str], optional
                :param disposition_action: Filters results by the retention policy with this disposition
        action., defaults to None
                :type disposition_action: Optional[GetFileVersionRetentionsDispositionAction], optional
                :param disposition_before: Filters results by files that will have their disposition
        come into effect before this date., defaults to None
                :type disposition_before: Optional[str], optional
                :param disposition_after: Filters results by files that will have their disposition
        come into effect after this date., defaults to None
                :type disposition_after: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'file_id': to_string(file_id),
                'file_version_id': to_string(file_version_id),
                'policy_id': to_string(policy_id),
                'disposition_action': to_string(disposition_action),
                'disposition_before': to_string(disposition_before),
                'disposition_after': to_string(disposition_after),
                'limit': to_string(limit),
                'marker': to_string(marker),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/file_version_retentions',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FileVersionRetentions)

    def get_file_version_retention_by_id(
        self,
        file_version_retention_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> FileVersionRetention:
        """
                Returns information about a file version retention.

                **Note**:


                File retention API is now **deprecated**.


                To get information about files and file versions under retention,


                see [files under retention](https://developer.box.com/reference/get-retention-policy-assignments-id-files-under-retention) or [file versions under retention](https://developer.box.com/reference/get-retention-policy-assignments-id-file-versions-under-retention) endpoints.

                :param file_version_retention_id: The ID of the file version retention.
        Example: "3424234"
                :type file_version_retention_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/file_version_retentions/',
                        to_string(file_version_retention_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, FileVersionRetention)



from enum import Enum

from typing import Optional

from typing import List

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.template_signer_input import TemplateSignerInput

from box_sdk_gen.box.errors import BoxSDKError


class TemplateSignerRoleField(str, Enum):
    SIGNER = 'signer'
    APPROVER = 'approver'
    FINAL_COPY_READER = 'final_copy_reader'


class TemplateSigner(BaseObject):
    def __init__(
        self,
        *,
        inputs: Optional[List[TemplateSignerInput]] = None,
        email: Optional[str] = None,
        role: Optional[TemplateSignerRoleField] = None,
        is_in_person: Optional[bool] = None,
        order: Optional[int] = None,
        signer_group_id: Optional[str] = None,
        label: Optional[str] = None,
        public_id: Optional[str] = None,
        is_password_required: Optional[bool] = None,
        is_phone_number_required: Optional[bool] = None,
        login_required: Optional[bool] = None,
        **kwargs
    ):
        """
                :param email: Email address of the signer., defaults to None
                :type email: Optional[str], optional
                :param role: Defines the role of the signer in the signature request. A role of
        `signer` needs to sign the document, a role `approver`
        approves the document and
        a `final_copy_reader` role only
        receives the final signed document and signing log., defaults to None
                :type role: Optional[TemplateSignerRoleField], optional
                :param is_in_person: Used in combination with an embed URL for a sender.
        After the sender signs, they will be
        redirected to the next `in_person` signer., defaults to None
                :type is_in_person: Optional[bool], optional
                :param order: Order of the signer., defaults to None
                :type order: Optional[int], optional
                :param signer_group_id: If provided, this value points signers that are assigned the same inputs and belongs to same signer group.
        A signer group is not a Box Group. It is an entity that belongs to the template itself and can only be used
        within Box Sign requests created from it., defaults to None
                :type signer_group_id: Optional[str], optional
                :param label: A placeholder label for the signer set by the template creator to differentiate between signers., defaults to None
                :type label: Optional[str], optional
                :param public_id: An identifier for the signer. This can be used to identify a signer within the template., defaults to None
                :type public_id: Optional[str], optional
                :param is_password_required: If true for signers with a defined email, the password provided when the template was created is used by default.
        If true for signers without a specified / defined email, the creator needs to provide a password when using the template., defaults to None
                :type is_password_required: Optional[bool], optional
                :param is_phone_number_required: If true for signers with a defined email, the phone number provided when the template was created is used by default.
        If true for signers without a specified / defined email, the template creator needs to provide a phone number when creating a request., defaults to None
                :type is_phone_number_required: Optional[bool], optional
                :param login_required: If true, the signer is required to login to access the document., defaults to None
                :type login_required: Optional[bool], optional
        """
        super().__init__(**kwargs)
        self.inputs = inputs
        self.email = email
        self.role = role
        self.is_in_person = is_in_person
        self.order = order
        self.signer_group_id = signer_group_id
        self.label = label
        self.public_id = public_id
        self.is_password_required = is_password_required
        self.is_phone_number_required = is_phone_number_required
        self.login_required = login_required



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.enterprise_base import EnterpriseBase

from box_sdk_gen.schemas.user_base import UserBase

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class ShieldInformationBarrierTypeField(str, Enum):
    SHIELD_INFORMATION_BARRIER = 'shield_information_barrier'


class ShieldInformationBarrierStatusField(str, Enum):
    DRAFT = 'draft'
    PENDING = 'pending'
    DISABLED = 'disabled'
    ENABLED = 'enabled'
    INVALID = 'invalid'


class ShieldInformationBarrier(BaseObject):
    _discriminator = 'type', {'shield_information_barrier'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[ShieldInformationBarrierTypeField] = None,
        enterprise: Optional[EnterpriseBase] = None,
        status: Optional[ShieldInformationBarrierStatusField] = None,
        created_at: Optional[DateTime] = None,
        created_by: Optional[UserBase] = None,
        updated_at: Optional[DateTime] = None,
        updated_by: Optional[UserBase] = None,
        enabled_at: Optional[DateTime] = None,
        enabled_by: Optional[UserBase] = None,
        **kwargs
    ):
        """
                :param id: The unique identifier for the shield information barrier., defaults to None
                :type id: Optional[str], optional
                :param type: The type of the shield information barrier., defaults to None
                :type type: Optional[ShieldInformationBarrierTypeField], optional
                :param enterprise: The `type` and `id` of enterprise this barrier is under., defaults to None
                :type enterprise: Optional[EnterpriseBase], optional
                :param status: Status of the shield information barrier., defaults to None
                :type status: Optional[ShieldInformationBarrierStatusField], optional
                :param created_at: ISO date time string when this
        shield information barrier object was created., defaults to None
                :type created_at: Optional[DateTime], optional
                :param created_by: The user who created this shield information barrier., defaults to None
                :type created_by: Optional[UserBase], optional
                :param updated_at: ISO date time string when this shield information barrier was updated., defaults to None
                :type updated_at: Optional[DateTime], optional
                :param updated_by: The user that updated this shield information barrier., defaults to None
                :type updated_by: Optional[UserBase], optional
                :param enabled_at: ISO date time string when this shield information barrier was enabled., defaults to None
                :type enabled_at: Optional[DateTime], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.enterprise = enterprise
        self.status = status
        self.created_at = created_at
        self.created_by = created_by
        self.updated_at = updated_at
        self.updated_by = updated_by
        self.enabled_at = enabled_at
        self.enabled_by = enabled_by



import threading
from enum import Enum
from typing import Optional, Generator

from ..box.errors import BoxSDKError
from ..schemas.events import Events
from ..schemas.event import Event
from ..schemas.realtime_server import RealtimeServer
from ..networking.fetch_options import FetchOptions, ResponseFormat
from ..networking.fetch_response import FetchResponse


class RealtimeServerEvent(str, Enum):
    NEW_CHANGE = 'new_change'
    RECONNECT = 'reconnect'


class EventStreamAction(str, Enum):
    FETCH_EVENTS = 'fetch_events'
    RECONNECT = 'reconnect'
    RETRY = 'retry'
    STOP = 'stop'


class EventStream:
    """
    EventStream is an iterator that fetches events from the Box API.
    It uses long polling to receive real-time updates.
    This class is designed to be used as a Python iterator.

    Example usage:
        events_stream = client.events.get_event_stream()
        for event in events_stream:
            print(event)
    """

    def __init__(self, *, events_manager, query_params, headers_input):
        """
        Initialize the EventStream.

        :param events_manager: The EventsManager instance which provides relevant methods to fetch events.
        :param query_params: The query parameters to use for fetching events.
        :param headers_input: The headers to include in the request.
        """
        self._events_manager = events_manager
        self._query_params = query_params
        self._headers_input = headers_input
        self._stream_position = query_params.stream_position or 'now'
        self._long_polling_info: Optional[RealtimeServer] = None
        self._long_polling_retries: int = 0
        self._started: bool = False
        self._stopped: bool = False
        self._stop_event = threading.Event()
        self._deduplication_size = 1000
        self._dedupHash = dict()

    def __iter__(self) -> Generator[Event, None, None]:
        """Make EventStream iterable. Yields Event objects."""
        return self._event_generator()

    def _event_generator(self) -> Generator[Event, None, None]:
        """Generator that yields Event objects from the stream."""
        if not self._started:
            self._started = True

        try:
            # Start with fetching events to get initial events and stream position
            yield from self._fetch_events()

            # Then start long polling loop
            while not self._stopped and not self._stop_event.is_set():
                try:
                    action = self._get_long_poll_info_and_poll()

                    if action == EventStreamAction.FETCH_EVENTS:
                        # Fetch new events when notified
                        yield from self._fetch_events()
                    elif action == EventStreamAction.RECONNECT:
                        # Continue the loop to get new long polling info
                        continue
                    elif action == EventStreamAction.RETRY:
                        # Wait a bit before retrying
                        if not self._stop_event.wait(5):
                            continue
                        else:
                            break
                    elif action == EventStreamAction.STOP:
                        break
                    else:
                        # Continue long polling
                        continue

                except Exception as e:
                    if not self._stopped and not self._stop_event.is_set():
                        # Wait a bit before retrying
                        if not self._stop_event.wait(5):
                            continue
                    break

        except Exception as e:
            if not self._stopped and not self._stop_event.is_set():
                pass
            return

    def stop(self):
        """Stop the event stream."""
        self._stopped = True
        self._stop_event.set()

    def _get_long_poll_info(self):
        """Fetch long polling info from the server."""
        if self._stopped or self._stop_event.is_set():
            return

        try:
            info = self._events_manager.get_events_with_long_polling()

            server = next(
                (e for e in info.entries or [] if e.type == 'realtime_server'), None
            )
            if not server:
                raise BoxSDKError(message='No realtime server found in the response.')

            self._long_polling_info = server
            self._long_polling_retries = 0

        except Exception as error:
            if not self._stopped and not self._stop_event.is_set():
                raise error

    def _get_long_poll_info_and_poll(self) -> str:
        """Get long polling info and perform a long poll, returning the action to take."""
        if self._stopped or self._stop_event.is_set():
            return 'stop'

        # Get long polling info if needed
        if not self._long_polling_info or self._long_polling_retries > int(
            self._long_polling_info.max_retries or '10'
        ):
            self._get_long_poll_info()

        return self._do_long_poll()

    def _do_long_poll(self) -> str:
        """Perform the long polling request and return action to take."""
        if self._stopped or self._stop_event.is_set():
            return EventStreamAction.STOP

        try:
            self._long_polling_retries += 1

            long_poll_url = self._long_polling_info.url
            separator = '&' if '?' in long_poll_url else '?'
            long_poll_with_stream_position = (
                f"{long_poll_url}{separator}stream_position={self._stream_position}"
            )

            response: FetchResponse = (
                self._events_manager.network_session.network_client.fetch(
                    FetchOptions(
                        url=long_poll_with_stream_position,
                        method='GET',
                        headers={
                            'Content-Type': 'application/json',
                        },
                        response_format=ResponseFormat.JSON,
                        auth=self._events_manager.auth,
                        network_session=self._events_manager.network_session,
                    )
                )
            )

            if self._stopped or self._stop_event.is_set():
                return EventStreamAction.STOP

            if response.data:
                message = response.data

                if isinstance(message, dict):
                    message_text = message.get('message', '')

                    if message_text == RealtimeServerEvent.NEW_CHANGE:
                        return EventStreamAction.FETCH_EVENTS
                    elif message_text == RealtimeServerEvent.RECONNECT:
                        return EventStreamAction.RECONNECT

                # Continue long polling
                return self._do_long_poll()

        except Exception as error:
            if not self._stopped and not self._stop_event.is_set():
                return 'retry'

        return 'stop'

    def _fetch_events(self) -> Generator[Event, None, None]:
        """Fetch events from the API and yield Event objects."""
        if self._stopped or self._stop_event.is_set():
            return

        try:
            # Prepare query parameters for the get_events call
            fetch_params = self._query_params.__dict__
            fetch_params['stream_position'] = self._stream_position

            # Add extra headers if provided
            if self._headers_input and self._headers_input.extra_headers:
                fetch_params['extra_headers'] = self._headers_input.extra_headers

            events: Events = self._events_manager.get_events(**fetch_params)

            # Update stream position for next request
            if events.next_stream_position is not None:
                self._stream_position = str(events.next_stream_position)
            else:
                self._stream_position = 'now'

            # Yield Event objects if any
            if events.entries:
                for event in events.entries:
                    event_id = event.event_id
                    if event_id not in self._dedupHash:
                        self._dedupHash[event_id] = True
                        if self._stopped or self._stop_event.is_set():
                            return
                        yield event

            if len(self._dedupHash) > self._deduplication_size:
                self._dedupHash.clear()
                event_ids = list(events.entries.map(lambda e: e.event_id))
                for event_id in event_ids:
                    self._dedupHash[event_id] = True

        except Exception as error:
            if not self._stopped and not self._stop_event.is_set():
                raise error



from enum import Enum

from typing import Optional

from typing import List

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.groups import Groups

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.group_full import GroupFull

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class CreateGroupInvitabilityLevel(str, Enum):
    ADMINS_ONLY = 'admins_only'
    ADMINS_AND_MEMBERS = 'admins_and_members'
    ALL_MANAGED_USERS = 'all_managed_users'


class CreateGroupMemberViewabilityLevel(str, Enum):
    ADMINS_ONLY = 'admins_only'
    ADMINS_AND_MEMBERS = 'admins_and_members'
    ALL_MANAGED_USERS = 'all_managed_users'


class UpdateGroupByIdInvitabilityLevel(str, Enum):
    ADMINS_ONLY = 'admins_only'
    ADMINS_AND_MEMBERS = 'admins_and_members'
    ALL_MANAGED_USERS = 'all_managed_users'


class UpdateGroupByIdMemberViewabilityLevel(str, Enum):
    ADMINS_ONLY = 'admins_only'
    ADMINS_AND_MEMBERS = 'admins_and_members'
    ALL_MANAGED_USERS = 'all_managed_users'


class GroupsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_groups(
        self,
        *,
        filter_term: Optional[str] = None,
        fields: Optional[List[str]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> Groups:
        """
                Retrieves all of the groups for a given enterprise. The user

                must have admin permissions to inspect enterprise's groups.

                :param filter_term: Limits the results to only groups whose `name` starts
        with the search term., defaults to None
                :type filter_term: Optional[str], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param offset: The offset of the item at which to begin the response.

        Queries with offset parameter value
        exceeding 10000 will be rejected
        with a 400 response., defaults to None
                :type offset: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'filter_term': to_string(filter_term),
                'fields': to_string(fields),
                'limit': to_string(limit),
                'offset': to_string(offset),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/groups']),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, Groups)

    def create_group(
        self,
        name: str,
        *,
        provenance: Optional[str] = None,
        external_sync_identifier: Optional[str] = None,
        description: Optional[str] = None,
        invitability_level: Optional[CreateGroupInvitabilityLevel] = None,
        member_viewability_level: Optional[CreateGroupMemberViewabilityLevel] = None,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> GroupFull:
        """
                Creates a new group of users in an enterprise. Only users with admin

                permissions can create new groups.

                :param name: The name of the new group to be created. This name must be unique
        within the enterprise.
                :type name: str
                :param provenance: Keeps track of which external source this group is
        coming, for example `Active Directory`, or `Okta`.

        Setting this will also prevent Box admins from editing
        the group name and its members directly via the Box
        web application.

        This is desirable for one-way syncing of groups., defaults to None
                :type provenance: Optional[str], optional
                :param external_sync_identifier: An arbitrary identifier that can be used by
        external group sync tools to link this Box Group to
        an external group.

        Example values of this field
        could be an **Active Directory Object ID** or a **Google
        Group ID**.

        We recommend you use of this field in
        order to avoid issues when group names are updated in
        either Box or external systems., defaults to None
                :type external_sync_identifier: Optional[str], optional
                :param description: A human readable description of the group., defaults to None
                :type description: Optional[str], optional
                :param invitability_level: Specifies who can invite the group to collaborate
        on folders.

        When set to `admins_only` the enterprise admin, co-admins,
        and the group's admin can invite the group.

        When set to `admins_and_members` all the admins listed
        above and group members can invite the group.

        When set to `all_managed_users` all managed users in the
        enterprise can invite the group., defaults to None
                :type invitability_level: Optional[CreateGroupInvitabilityLevel], optional
                :param member_viewability_level: Specifies who can see the members of the group.

        * `admins_only` - the enterprise admin, co-admins, group's
          group admin.
        * `admins_and_members` - all admins and group members.
        * `all_managed_users` - all managed users in the
          enterprise., defaults to None
                :type member_viewability_level: Optional[CreateGroupMemberViewabilityLevel], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'provenance': provenance,
            'external_sync_identifier': external_sync_identifier,
            'description': description,
            'invitability_level': invitability_level,
            'member_viewability_level': member_viewability_level,
        }
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join([self.network_session.base_urls.base_url, '/2.0/groups']),
                method='POST',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, GroupFull)

    def get_group_by_id(
        self,
        group_id: str,
        *,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> GroupFull:
        """
                Retrieves information about a group. Only members of this

                group or users with admin-level permissions will be able to


                use this API.

                :param group_id: The ID of the group.
        Example: "57645"
                :type group_id: str
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/groups/',
                        to_string(group_id),
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, GroupFull)

    def update_group_by_id(
        self,
        group_id: str,
        *,
        name: Optional[str] = None,
        provenance: Optional[str] = None,
        external_sync_identifier: Optional[str] = None,
        description: Optional[str] = None,
        invitability_level: Optional[UpdateGroupByIdInvitabilityLevel] = None,
        member_viewability_level: Optional[
            UpdateGroupByIdMemberViewabilityLevel
        ] = None,
        fields: Optional[List[str]] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> GroupFull:
        """
                Updates a specific group. Only admins of this

                group or users with admin-level permissions will be able to


                use this API.

                :param group_id: The ID of the group.
        Example: "57645"
                :type group_id: str
                :param name: The name of the new group to be created. Must be unique within the
        enterprise., defaults to None
                :type name: Optional[str], optional
                :param provenance: Keeps track of which external source this group is
        coming, for example `Active Directory`, or `Okta`.

        Setting this will also prevent Box admins from editing
        the group name and its members directly via the Box
        web application.

        This is desirable for one-way syncing of groups., defaults to None
                :type provenance: Optional[str], optional
                :param external_sync_identifier: An arbitrary identifier that can be used by
        external group sync tools to link this Box Group to
        an external group.

        Example values of this field
        could be an **Active Directory Object ID** or a **Google
        Group ID**.

        We recommend you use of this field in
        order to avoid issues when group names are updated in
        either Box or external systems., defaults to None
                :type external_sync_identifier: Optional[str], optional
                :param description: A human readable description of the group., defaults to None
                :type description: Optional[str], optional
                :param invitability_level: Specifies who can invite the group to collaborate
        on folders.

        When set to `admins_only` the enterprise admin, co-admins,
        and the group's admin can invite the group.

        When set to `admins_and_members` all the admins listed
        above and group members can invite the group.

        When set to `all_managed_users` all managed users in the
        enterprise can invite the group., defaults to None
                :type invitability_level: Optional[UpdateGroupByIdInvitabilityLevel], optional
                :param member_viewability_level: Specifies who can see the members of the group.

        * `admins_only` - the enterprise admin, co-admins, group's
          group admin.
        * `admins_and_members` - all admins and group members.
        * `all_managed_users` - all managed users in the
          enterprise., defaults to None
                :type member_viewability_level: Optional[UpdateGroupByIdMemberViewabilityLevel], optional
                :param fields: A comma-separated list of attributes to include in the
        response. This can be used to request fields that are
        not normally returned in a standard response.

        Be aware that specifying this parameter will have the
        effect that none of the standard fields are returned in
        the response unless explicitly specified, instead only
        fields for the mini representation are returned, additional
        to the fields requested., defaults to None
                :type fields: Optional[List[str]], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'name': name,
            'provenance': provenance,
            'external_sync_identifier': external_sync_identifier,
            'description': description,
            'invitability_level': invitability_level,
            'member_viewability_level': member_viewability_level,
        }
        query_params_map: Dict[str, str] = prepare_params({'fields': to_string(fields)})
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/groups/',
                        to_string(group_id),
                    ]
                ),
                method='PUT',
                params=query_params_map,
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, GroupFull)

    def delete_group_by_id(
        self, group_id: str, *, extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Permanently deletes a group. Only users with

                admin-level permissions will be able to use this API.

                :param group_id: The ID of the group.
        Example: "57645"
                :type group_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/groups/',
                        to_string(group_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None



from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from box_sdk_gen.schemas.v2025_r0.shield_list_content_country_v2025_r0 import (
    ShieldListContentCountryV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_domain_v2025_r0 import (
    ShieldListContentDomainV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_email_v2025_r0 import (
    ShieldListContentEmailV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_ip_v2025_r0 import (
    ShieldListContentIpV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_integration_v2025_r0 import (
    ShieldListContentIntegrationV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.enterprise_reference_v2025_r0 import (
    EnterpriseReferenceV2025R0,
)

from box_sdk_gen.schemas.v2025_r0.shield_list_content_v2025_r0 import (
    ShieldListContentV2025R0,
)

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class ShieldListV2025R0(BaseObject):
    def __init__(
        self,
        id: str,
        type: str,
        name: str,
        enterprise: EnterpriseReferenceV2025R0,
        created_at: DateTime,
        updated_at: DateTime,
        content: ShieldListContentV2025R0,
        *,
        description: Optional[str] = None,
        **kwargs
    ):
        """
        :param id: Unique identifier for the shield list.
        :type id: str
        :param type: Type of the object.
        :type type: str
        :param name: Name of the shield list.
        :type name: str
        :param created_at: ISO date time string when this shield list object was created.
        :type created_at: DateTime
        :param updated_at: ISO date time string when this shield list object was updated.
        :type updated_at: DateTime
        :param description: Description of Shield List., defaults to None
        :type description: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.name = name
        self.enterprise = enterprise
        self.created_at = created_at
        self.updated_at = updated_at
        self.content = content
        self.description = description



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.shield_information_barrier_segment_member import (
    ShieldInformationBarrierSegmentMember,
)

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.shield_information_barrier_segment_members import (
    ShieldInformationBarrierSegmentMembers,
)

from box_sdk_gen.schemas.shield_information_barrier_base import (
    ShieldInformationBarrierBase,
)

from box_sdk_gen.schemas.user_base import UserBase

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class CreateShieldInformationBarrierSegmentMemberType(str, Enum):
    SHIELD_INFORMATION_BARRIER_SEGMENT_MEMBER = (
        'shield_information_barrier_segment_member'
    )


class CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField(
    str, Enum
):
    SHIELD_INFORMATION_BARRIER_SEGMENT = 'shield_information_barrier_segment'


class CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegment(
    BaseObject
):
    _discriminator = 'type', {'shield_information_barrier_segment'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[
            CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField
        ] = None,
        **kwargs
    ):
        """
                :param id: The ID reference of the
        requesting shield information barrier segment., defaults to None
                :type id: Optional[str], optional
                :param type: The type of the shield barrier segment for this member., defaults to None
                :type type: Optional[CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class ShieldInformationBarrierSegmentMembersManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_shield_information_barrier_segment_member_by_id(
        self,
        shield_information_barrier_segment_member_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldInformationBarrierSegmentMember:
        """
                Retrieves a shield information barrier

                segment member by its ID.

                :param shield_information_barrier_segment_member_id: The ID of the shield information barrier segment Member.
        Example: "7815"
                :type shield_information_barrier_segment_member_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_information_barrier_segment_members/',
                        to_string(shield_information_barrier_segment_member_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldInformationBarrierSegmentMember)

    def delete_shield_information_barrier_segment_member_by_id(
        self,
        shield_information_barrier_segment_member_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Deletes a shield information barrier

                segment member based on provided ID.

                :param shield_information_barrier_segment_member_id: The ID of the shield information barrier segment Member.
        Example: "7815"
                :type shield_information_barrier_segment_member_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_information_barrier_segment_members/',
                        to_string(shield_information_barrier_segment_member_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None

    def get_shield_information_barrier_segment_members(
        self,
        shield_information_barrier_segment_id: str,
        *,
        marker: Optional[str] = None,
        limit: Optional[int] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldInformationBarrierSegmentMembers:
        """
                Lists shield information barrier segment members

                based on provided segment IDs.

                :param shield_information_barrier_segment_id: The ID of the shield information barrier segment.
                :type shield_information_barrier_segment_id: str
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param limit: The maximum number of items to return per page., defaults to None
                :type limit: Optional[int], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'shield_information_barrier_segment_id': to_string(
                    shield_information_barrier_segment_id
                ),
                'marker': to_string(marker),
                'limit': to_string(limit),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_information_barrier_segment_members',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldInformationBarrierSegmentMembers)

    def create_shield_information_barrier_segment_member(
        self,
        shield_information_barrier_segment: CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegment,
        user: UserBase,
        *,
        type: Optional[CreateShieldInformationBarrierSegmentMemberType] = None,
        shield_information_barrier: Optional[ShieldInformationBarrierBase] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> ShieldInformationBarrierSegmentMember:
        """
                Creates a new shield information barrier segment member.
                :param shield_information_barrier_segment: The `type` and `id` of the
        requested shield information barrier segment.
                :type shield_information_barrier_segment: CreateShieldInformationBarrierSegmentMemberShieldInformationBarrierSegment
                :param user: User to which restriction will be applied.
                :type user: UserBase
                :param type: A type of the shield barrier segment member., defaults to None
                :type type: Optional[CreateShieldInformationBarrierSegmentMemberType], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'type': type,
            'shield_information_barrier': shield_information_barrier,
            'shield_information_barrier_segment': shield_information_barrier_segment,
            'user': user,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/shield_information_barrier_segment_members',
                    ]
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, ShieldInformationBarrierSegmentMember)



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class ZipDownloadStatusStateField(str, Enum):
    IN_PROGRESS = 'in_progress'
    FAILED = 'failed'
    SUCCEEDED = 'succeeded'


class ZipDownloadStatus(BaseObject):
    def __init__(
        self,
        *,
        total_file_count: Optional[int] = None,
        downloaded_file_count: Optional[int] = None,
        skipped_file_count: Optional[int] = None,
        skipped_folder_count: Optional[int] = None,
        state: Optional[ZipDownloadStatusStateField] = None,
        **kwargs
    ):
        """
                :param total_file_count: The total number of files in the archive., defaults to None
                :type total_file_count: Optional[int], optional
                :param downloaded_file_count: The number of files that have already been downloaded., defaults to None
                :type downloaded_file_count: Optional[int], optional
                :param skipped_file_count: The number of files that have been skipped as they could not be
        downloaded. In many cases this is due to permission issues that have
        surfaced between the creation of the request for the archive and the
        archive being downloaded., defaults to None
                :type skipped_file_count: Optional[int], optional
                :param skipped_folder_count: The number of folders that have been skipped as they could not be
        downloaded. In many cases this is due to permission issues that have
        surfaced between the creation of the request for the archive and the
        archive being downloaded., defaults to None
                :type skipped_folder_count: Optional[int], optional
                :param state: The state of the archive being downloaded., defaults to None
                :type state: Optional[ZipDownloadStatusStateField], optional
        """
        super().__init__(**kwargs)
        self.total_file_count = total_file_count
        self.downloaded_file_count = downloaded_file_count
        self.skipped_file_count = skipped_file_count
        self.skipped_folder_count = skipped_folder_count
        self.state = state



from enum import Enum

from box_sdk_gen.internal.base_object import BaseObject

from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.storage_policy_assignments import StoragePolicyAssignments

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.storage_policy_assignment import StoragePolicyAssignment

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class GetStoragePolicyAssignmentsResolvedForType(str, Enum):
    USER = 'user'
    ENTERPRISE = 'enterprise'


class CreateStoragePolicyAssignmentStoragePolicyTypeField(str, Enum):
    STORAGE_POLICY = 'storage_policy'


class CreateStoragePolicyAssignmentStoragePolicy(BaseObject):
    _discriminator = 'type', {'storage_policy'}

    def __init__(
        self,
        id: str,
        *,
        type: CreateStoragePolicyAssignmentStoragePolicyTypeField = CreateStoragePolicyAssignmentStoragePolicyTypeField.STORAGE_POLICY,
        **kwargs
    ):
        """
        :param id: The ID of the storage policy to assign.
        :type id: str
        :param type: The type to assign., defaults to CreateStoragePolicyAssignmentStoragePolicyTypeField.STORAGE_POLICY
        :type type: CreateStoragePolicyAssignmentStoragePolicyTypeField, optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class CreateStoragePolicyAssignmentAssignedToTypeField(str, Enum):
    USER = 'user'
    ENTERPRISE = 'enterprise'


class CreateStoragePolicyAssignmentAssignedTo(BaseObject):
    _discriminator = 'type', {'user', 'enterprise'}

    def __init__(
        self, type: CreateStoragePolicyAssignmentAssignedToTypeField, id: str, **kwargs
    ):
        """
        :param type: The type to assign the policy to.
        :type type: CreateStoragePolicyAssignmentAssignedToTypeField
        :param id: The ID of the user or enterprise.
        :type id: str
        """
        super().__init__(**kwargs)
        self.type = type
        self.id = id


class UpdateStoragePolicyAssignmentByIdStoragePolicyTypeField(str, Enum):
    STORAGE_POLICY = 'storage_policy'


class UpdateStoragePolicyAssignmentByIdStoragePolicy(BaseObject):
    _discriminator = 'type', {'storage_policy'}

    def __init__(
        self,
        id: str,
        *,
        type: UpdateStoragePolicyAssignmentByIdStoragePolicyTypeField = UpdateStoragePolicyAssignmentByIdStoragePolicyTypeField.STORAGE_POLICY,
        **kwargs
    ):
        """
        :param id: The ID of the storage policy to assign.
        :type id: str
        :param type: The type to assign., defaults to UpdateStoragePolicyAssignmentByIdStoragePolicyTypeField.STORAGE_POLICY
        :type type: UpdateStoragePolicyAssignmentByIdStoragePolicyTypeField, optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class StoragePolicyAssignmentsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_storage_policy_assignments(
        self,
        resolved_for_type: GetStoragePolicyAssignmentsResolvedForType,
        resolved_for_id: str,
        *,
        marker: Optional[str] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> StoragePolicyAssignments:
        """
                Fetches all the storage policy assignment for an enterprise or user.
                :param resolved_for_type: The target type to return assignments for.
                :type resolved_for_type: GetStoragePolicyAssignmentsResolvedForType
                :param resolved_for_id: The ID of the user or enterprise to return assignments for.
                :type resolved_for_id: str
                :param marker: Defines the position marker at which to begin returning results. This is
        used when paginating using marker-based pagination.

        This requires `usemarker` to be set to `true`., defaults to None
                :type marker: Optional[str], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        query_params_map: Dict[str, str] = prepare_params(
            {
                'marker': to_string(marker),
                'resolved_for_type': to_string(resolved_for_type),
                'resolved_for_id': to_string(resolved_for_id),
            }
        )
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/storage_policy_assignments',
                    ]
                ),
                method='GET',
                params=query_params_map,
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, StoragePolicyAssignments)

    def create_storage_policy_assignment(
        self,
        storage_policy: CreateStoragePolicyAssignmentStoragePolicy,
        assigned_to: CreateStoragePolicyAssignmentAssignedTo,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> StoragePolicyAssignment:
        """
                Creates a storage policy assignment for an enterprise or user.
                :param storage_policy: The storage policy to assign to the user or
        enterprise.
                :type storage_policy: CreateStoragePolicyAssignmentStoragePolicy
                :param assigned_to: The user or enterprise to assign the storage
        policy to.
                :type assigned_to: CreateStoragePolicyAssignmentAssignedTo
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {
            'storage_policy': storage_policy,
            'assigned_to': assigned_to,
        }
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/storage_policy_assignments',
                    ]
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, StoragePolicyAssignment)

    def get_storage_policy_assignment_by_id(
        self,
        storage_policy_assignment_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> StoragePolicyAssignment:
        """
                Fetches a specific storage policy assignment.
                :param storage_policy_assignment_id: The ID of the storage policy assignment.
        Example: "932483"
                :type storage_policy_assignment_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/storage_policy_assignments/',
                        to_string(storage_policy_assignment_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, StoragePolicyAssignment)

    def update_storage_policy_assignment_by_id(
        self,
        storage_policy_assignment_id: str,
        storage_policy: UpdateStoragePolicyAssignmentByIdStoragePolicy,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> StoragePolicyAssignment:
        """
                Updates a specific storage policy assignment.
                :param storage_policy_assignment_id: The ID of the storage policy assignment.
        Example: "932483"
                :type storage_policy_assignment_id: str
                :param storage_policy: The storage policy to assign to the user or
        enterprise.
                :type storage_policy: UpdateStoragePolicyAssignmentByIdStoragePolicy
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'storage_policy': storage_policy}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/storage_policy_assignments/',
                        to_string(storage_policy_assignment_id),
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, StoragePolicyAssignment)

    def delete_storage_policy_assignment_by_id(
        self,
        storage_policy_assignment_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Delete a storage policy assignment.

                Deleting a storage policy assignment on a user


                will have the user inherit the enterprise's default


                storage policy.


                There is a rate limit for calling this endpoint of only


                twice per user in a 24 hour time frame.

                :param storage_policy_assignment_id: The ID of the storage policy assignment.
        Example: "932483"
                :type storage_policy_assignment_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/storage_policy_assignments/',
                        to_string(storage_policy_assignment_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None



from enum import Enum

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class IntegrationMappingBaseTypeField(str, Enum):
    INTEGRATION_MAPPING = 'integration_mapping'


class IntegrationMappingBase(BaseObject):
    _discriminator = 'type', {'integration_mapping'}

    def __init__(
        self,
        id: str,
        *,
        type: IntegrationMappingBaseTypeField = IntegrationMappingBaseTypeField.INTEGRATION_MAPPING,
        **kwargs
    ):
        """
                :param id: A unique identifier of a folder mapping
        (part of a composite key together
        with `integration_type`).
                :type id: str
                :param type: Mapping type., defaults to IntegrationMappingBaseTypeField.INTEGRATION_MAPPING
                :type type: IntegrationMappingBaseTypeField, optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type



from typing import Dict

from box_sdk_gen.serialization.json import SerializedData

from box_sdk_gen.internal.utils import sanitize_map

from box_sdk_gen.serialization.json import sanitize_serialized_data


class DataSanitizer:
    def __init__(self):
        self._keys_to_sanitize = {
            'authorization': '',
            'access_token': '',
            'refresh_token': '',
            'subject_token': '',
            'token': '',
            'client_id': '',
            'client_secret': '',
            'shared_link': '',
            'download_url': '',
            'jwt_private_key': '',
            'jwt_private_key_passphrase': '',
            'password': '',
        }

    def sanitize_headers(self, headers: Dict[str, str]) -> Dict[str, str]:
        return sanitize_map(headers, self._keys_to_sanitize)

    def sanitize_body(self, body: SerializedData) -> SerializedData:
        return sanitize_serialized_data(body, self._keys_to_sanitize)



from typing import Union

from box_sdk_gen.schemas.keyword_skill_card import KeywordSkillCard

from box_sdk_gen.schemas.timeline_skill_card import TimelineSkillCard

from box_sdk_gen.schemas.transcript_skill_card import TranscriptSkillCard

from box_sdk_gen.schemas.status_skill_card import StatusSkillCard

from box_sdk_gen.box.errors import BoxSDKError

SkillCard = Union[
    KeywordSkillCard, TimelineSkillCard, TranscriptSkillCard, StatusSkillCard
]



import base64
import datetime
import hashlib
import os
import re
import shutil
import uuid
import time
import hmac
from random import uniform
from enum import Enum
from io import SEEK_CUR, SEEK_END, SEEK_SET, BufferedIOBase, BytesIO
from typing import Any, Callable, Dict, Iterable, Optional, TypeVar, BinaryIO

from abc import abstractmethod
from typing import Any

try:
    import jwt
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives import serialization
except ImportError:
    jwt, default_backend, serialization = None, None, None

from .base_object import BaseObject
from ..serialization.json import sd_to_json, sanitized_value
from ..serialization.json import serialize
from .null_value import null

ByteStream = BufferedIOBase
OutputStream = BinaryIO
Buffer = bytes


class ResponseByteStream(ByteStream):
    def __init__(self, request_iterator):
        self._iterator = request_iterator
        self._buffer = b''
        self._position = 0
        self._eos = False

    def _read_from_iterator(self, size):
        """
        Read up to `size` bytes from the iterator into the buffer
        :param size: Number of bytes to read. If None, read the entire stream.
        """
        if self._eos:
            return

        while len(self._buffer) < size:
            try:
                chunk = next(self._iterator)
                self._buffer += chunk
            except StopIteration:
                self._eos = True
                break

    def tell(self):
        """
        Returns the current position in the stream
        :return:
        """
        return self._position

    def read(self, size=None):
        """
        Reads up to `size` bytes from the stream
        :param size: Read up to `size` bytes from the stream. If None read the entire stream.
        :return: Bytes read from the stream
        """
        if size is None:
            # Read everything remaining in the stream.
            result = self._buffer + b''.join(self._iterator)
            self._buffer = b''
            self._position += len(result)
            self._eos = True
            return result

        self._read_from_iterator(size)
        result = self._buffer[:size]
        self._buffer = self._buffer[size:]
        self._position += len(result)
        return result

    def seek(self, position, whence=SEEK_SET):
        """
        Move the stream to given position
        :param position: Position to move to
        :param whence: One of SEEK_SET = 0, SEEK_CUR = 1 or SEEK_END = 2
        :return: The new position in the stream
        """
        if whence == SEEK_SET:
            if position < self._position:
                raise ValueError('Cannot seek backwards in a stream')
            self.read(position - self._position)
        elif whence == SEEK_CUR:
            self.read(position)
        elif whence == SEEK_END:
            raise NotImplementedError('SEEK_END is not supported for streams')
        else:
            raise ValueError('Invalid value for `whence`')

        return self._position


def get_env_var(name: str) -> str:
    return os.getenv(name)


def get_uuid() -> str:
    return str(uuid.uuid1())


def decode_base_64(value: str) -> str:
    return base64.b64decode(value).decode()


def generate_byte_buffer(size: int) -> Buffer:
    return Buffer(os.urandom(size))


def generate_byte_stream_from_buffer(buffer: Buffer) -> ByteStream:
    return BytesIO(buffer)


def generate_byte_stream(size: int) -> ByteStream:
    return BytesIO(os.urandom(size))


def buffer_equals(buffer1: Buffer, buffer2: Buffer) -> bool:
    return buffer1 == buffer2


def buffer_length(buffer: Buffer) -> int:
    return len(buffer)


def decode_base_64_byte_stream(value: str) -> ByteStream:
    return BytesIO(base64.b64decode(value))


def string_to_byte_stream(value: str) -> ByteStream:
    return BytesIO(bytes(value, 'utf-8'))


def read_byte_stream(byte_stream: ByteStream) -> Buffer:
    return Buffer(byte_stream.read())


def write_input_stream_to_output_stream(
    input_stream: ByteStream, output_stream: OutputStream
):
    shutil.copyfileobj(input_stream, output_stream)


def get_file_output_stream(file_path: str) -> OutputStream:
    return open(file_path, 'wb')


def close_file_output_stream(file_output_stream: OutputStream):
    file_output_stream.close()


def read_buffer_from_file(path: str) -> bytes:
    with open(path, 'rb') as file:
        return file.read()


def prepare_params(map: Dict[str, Optional[str]]) -> Dict[str, str]:
    return {k: v for k, v in map.items() if v is not None}


def to_string(value: Any) -> Optional[str]:
    if value is None:
        return None
    if isinstance(value, datetime.datetime):
        return date_time_to_string(value)
    if isinstance(value, datetime.date):
        return date_to_string(value)
    if (
        isinstance(value, BaseObject)
        or isinstance(value, list)
        and len(value) >= 1
        and isinstance(value[0], BaseObject)
    ):
        return ''.join(sd_to_json(serialize(value)).split())
    if isinstance(value, list):
        return ','.join(map(to_string, value))
    if isinstance(value, Enum):
        return value.value
    return str(value)


class HashName(str, Enum):
    SHA1 = 'sha1'


class Hash:
    def __init__(self, algorithm: HashName):
        self.algorithm = algorithm
        self.hash = hashlib.sha1()

    def update_hash(self, data: Buffer):
        self.hash.update(data)

    def digest_hash(self, encoding):
        return base64.b64encode(self.hash.digest()).decode("utf-8")


def hex_to_base_64(data: hex):
    return base64.b64encode(bytes.fromhex(data)).decode()


T = TypeVar('T')
Iterator = Iterable[T]
Accumulator = TypeVar('Accumulator')


def iterate_chunks(
    stream: ByteStream, chunk_size: int, file_size: int
) -> Iterable[ByteStream]:
    stream_is_finished = False
    while not stream_is_finished:
        copied_length = 0
        chunk = b''
        while copied_length < chunk_size:
            bytes_read = stream.read(chunk_size - copied_length)
            if bytes_read is None:
                # stream returns none when no bytes are ready currently but there are
                # potentially more bytes in the stream to be read.
                continue
            if not bytes_read:
                # stream is exhausted.
                stream_is_finished = True
                break
            chunk += bytes_read
            copied_length += len(bytes_read)
        if chunk:
            yield BytesIO(chunk)


def reduce_iterator(
    iterator: Iterator,
    reducer: Callable[[Accumulator, T], Accumulator],
    initial_value: Accumulator,
) -> Accumulator:
    result = initial_value

    for item in iterator:
        result = reducer(result, item)

    return result


def read_text_from_file(file_path: str) -> str:
    with open(file_path, 'r') as file:
        return file.read()


def is_browser() -> bool:
    return False


def get_epoch_time_in_seconds() -> int:
    return int(time.time())


def get_value_from_object_raw_data(obj: BaseObject, key: str) -> Any:
    keys = key.split('.')
    value: dict = obj.raw_data
    for k in keys:
        value = value.get(k, {})
    return value


class PrivateKeyDecryptor:
    """Class used for private key decryption in JWT auth."""

    @abstractmethod
    def decrypt_private_key(self, encryptedPrivateKey: str, passphrase: str) -> Any:
        """Decrypts private key using a passphrase."""
        pass


class DefaultPrivateKeyDecryptor(PrivateKeyDecryptor):
    def decrypt_private_key(self, encryptedPrivateKey: str, passphrase: str) -> Any:
        if default_backend is None or serialization is None:
            raise ImportError(
                'Missing `cryptography` dependency. `cryptography` library is required to create JWT assertion.'
            )
        encoded_private_key = encode_str_ascii_or_raise(encryptedPrivateKey)
        encoded_passphrase = encode_str_ascii_or_raise(passphrase)

        return serialization.load_pem_private_key(
            encoded_private_key,
            password=encoded_passphrase,
            backend=default_backend(),
        )


class JwtAlgorithm(str, Enum):
    HS256 = 'HS256'
    HS384 = 'HS384'
    HS512 = 'HS512'
    RS256 = 'RS256'
    RS384 = 'RS384'
    RS512 = 'RS512'
    ES256 = 'ES256'
    ES384 = 'ES384'
    ES512 = 'ES512'
    PS256 = 'PS256'
    PS384 = 'PS384'
    PS512 = 'PS512'
    none = 'none'


class JwtSignOptions(BaseObject):
    def __init__(
        self,
        algorithm: JwtAlgorithm,
        headers: Dict[str, str] = None,
        audience: Optional[str] = None,
        issuer: Optional[str] = None,
        subject: Optional[str] = None,
        jwtid: Optional[str] = None,
        keyid: Optional[str] = None,
        private_key_decryptor: Optional[PrivateKeyDecryptor] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        if headers is None:
            headers = {}
        self.algorithm = algorithm
        self.headers = headers
        self.audience = audience
        self.issuer = issuer
        self.subject = subject
        self.jwtid = jwtid
        self.keyid = keyid
        self.private_key_decryptor = (
            private_key_decryptor
            if private_key_decryptor is not None
            else DefaultPrivateKeyDecryptor()
        )


class JwtKey(BaseObject):
    def __init__(self, key: str, passphrase: str, **kwargs):
        super().__init__(**kwargs)
        self.key = key
        self.passphrase = passphrase


def encode_str_ascii_or_raise(passphrase: str) -> bytes:
    try:
        return passphrase.encode('ascii')
    except UnicodeError as unicode_error:
        raise TypeError(
            "private_key and private_key_passphrase must contain binary data (bytes/str), not a text/unicode string"
        ) from unicode_error


def create_jwt_assertion(claims: dict, key: JwtKey, options: JwtSignOptions) -> str:
    if jwt is None:
        raise ImportError(
            'Missing `PyJWT` dependency. `PyJWT` library is required to create JWT assertion.'
        )
    return jwt.encode(
        {
            'iss': options.issuer,
            'sub': options.subject,
            'box_sub_type': claims['box_sub_type'],
            'aud': options.audience,
            'jti': options.jwtid,
            'exp': claims['exp'],
        },
        options.private_key_decryptor.decrypt_private_key(key.key, key.passphrase),
        algorithm=options.algorithm,
        headers={'kid': options.keyid},
    )


Date = datetime.date
DateTime = datetime.datetime


def date_to_string(date: Date) -> str:
    return date.isoformat()


def date_from_string(date: str) -> Date:
    return Date.fromisoformat(date)


def date_time_to_string(date_time: DateTime) -> str:
    return date_time.isoformat().replace('+00:00', 'Z')


def date_time_from_string(date_time: str) -> DateTime:
    return DateTime.fromisoformat(date_time.replace('Z', '+00:00'))


def date_time_to_epoch_seconds(date_time: DateTime) -> int:
    return int(date_time.timestamp())


def epoch_seconds_to_date_time(epoch_seconds: int) -> DateTime:
    return DateTime.fromtimestamp(epoch_seconds, datetime.timezone.utc)


def delay_in_seconds(seconds: int):
    time.sleep(seconds)


def create_null():
    return null


def escape_unicode(value: str) -> str:
    def replace_char(match):
        char = match.group(0)
        code_point = ord(char)
        if char == '\n':
            return '\\n'
        elif char == '\r':
            return '\\r'
        elif char == '\t':
            return '\\t'
        elif code_point <= 0xFFFF:  # Basic Multilingual Plane (BMP)
            return f"\\u{code_point:04x}"
        else:  # Supplementary Plane (Surrogate Pair)
            code_point -= 0x10000
            high_surrogate = 0xD800 + (code_point >> 10)
            low_surrogate = 0xDC00 + (code_point & 0x3FF)
            return f"\\u{high_surrogate:04x}\\u{low_surrogate:04x}"

    # Replace any backslashes that are NOT part of a \/ with double backslash
    temp = re.sub(r'\\(?!/)', r'\\\\', value)

    # Match special characters, non-ASCII characters
    return re.sub(r'[^\x20-\x7e]|[\n\r\t]', replace_char, temp)


def compute_webhook_signature(
    body: str,
    headers: Dict[str, str],
    signature_key: str,
    escape_body: Optional[bool] = False,
) -> Optional[str]:
    """
    Computes the Hmac for the webhook notification given one signature key.

    :param body:
        The encoded webhook body.
    :param headers:
        The headers for the `Webhook` notification.
    :param signature_key:
        The `Webhook` signature key for this application.
    :param escape_body:
        Indicates if payload should be escaped or left as is.
    :return:
        An Hmac signature.
    """
    if signature_key is None:
        return None
    if headers.get('box-signature-version') != '1':
        return None
    if headers.get('box-signature-algorithm') != 'HmacSHA256':
        return None

    encoded_body = (escape_unicode(body) if escape_body else body).encode('utf-8')
    encoded_signature_key = signature_key.encode('utf-8')
    encoded_delivery_time_stamp = headers.get('box-delivery-timestamp').encode('utf-8')
    new_hmac = hmac.new(encoded_signature_key, digestmod=hashlib.sha256)
    new_hmac.update(encoded_body)
    new_hmac.update(encoded_delivery_time_stamp)
    signature = base64.b64encode(new_hmac.digest()).decode()
    return signature


def compare_signatures(
    expected_signature: Optional[str], received_signature: Optional[str]
) -> bool:
    if not expected_signature or not received_signature:
        return False
    if len(expected_signature) != len(received_signature):
        return False
    return hmac.compare_digest(expected_signature, received_signature)


def random(min: float, max: float) -> float:
    return uniform(min, max)


def sanitize_map(
    dictionary: Dict[str, str], keys_to_sanitize: Dict[str, str]
) -> Dict[str, str]:
    return {
        k: sanitized_value() if k.lower() in keys_to_sanitize else v
        for k, v in dictionary.items()
    }



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.legal_hold_policy_mini import LegalHoldPolicyMiniTypeField

from box_sdk_gen.schemas.legal_hold_policy_mini import LegalHoldPolicyMini

from box_sdk_gen.schemas.user_mini import UserMini

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class LegalHoldPolicyStatusField(str, Enum):
    ACTIVE = 'active'
    APPLYING = 'applying'
    RELEASING = 'releasing'
    RELEASED = 'released'


class LegalHoldPolicyAssignmentCountsField(BaseObject):
    def __init__(
        self,
        *,
        user: Optional[int] = None,
        folder: Optional[int] = None,
        file: Optional[int] = None,
        file_version: Optional[int] = None,
        ownership: Optional[int] = None,
        interactions: Optional[int] = None,
        **kwargs
    ):
        """
        :param user: The number of users this policy is applied to with the `access` type assignment., defaults to None
        :type user: Optional[int], optional
        :param folder: The number of folders this policy is applied to., defaults to None
        :type folder: Optional[int], optional
        :param file: The number of files this policy is applied to., defaults to None
        :type file: Optional[int], optional
        :param file_version: The number of file versions this policy is applied to., defaults to None
        :type file_version: Optional[int], optional
        :param ownership: The number of users this policy is applied to with the `ownership` type assignment., defaults to None
        :type ownership: Optional[int], optional
        :param interactions: The number of users this policy is applied to with the `interactions` type assignment., defaults to None
        :type interactions: Optional[int], optional
        """
        super().__init__(**kwargs)
        self.user = user
        self.folder = folder
        self.file = file
        self.file_version = file_version
        self.ownership = ownership
        self.interactions = interactions


class LegalHoldPolicy(LegalHoldPolicyMini):
    def __init__(
        self,
        id: str,
        *,
        policy_name: Optional[str] = None,
        description: Optional[str] = None,
        status: Optional[LegalHoldPolicyStatusField] = None,
        assignment_counts: Optional[LegalHoldPolicyAssignmentCountsField] = None,
        created_by: Optional[UserMini] = None,
        created_at: Optional[DateTime] = None,
        modified_at: Optional[DateTime] = None,
        deleted_at: Optional[DateTime] = None,
        filter_started_at: Optional[DateTime] = None,
        filter_ended_at: Optional[DateTime] = None,
        release_notes: Optional[str] = None,
        type: LegalHoldPolicyMiniTypeField = LegalHoldPolicyMiniTypeField.LEGAL_HOLD_POLICY,
        **kwargs
    ):
        """
                :param id: The unique identifier for this legal hold policy.
                :type id: str
                :param policy_name: Name of the legal hold policy., defaults to None
                :type policy_name: Optional[str], optional
                :param description: Description of the legal hold policy. Optional
        property with a 500 character limit., defaults to None
                :type description: Optional[str], optional
                :param status: Possible values:
        * 'active' - the policy is not in a transition state.
        * 'applying' - that the policy is in the process of
          being applied.
        * 'releasing' - that the process is in the process
          of being released.
        * 'released' - the policy is no longer active., defaults to None
                :type status: Optional[LegalHoldPolicyStatusField], optional
                :param assignment_counts: Counts of assignments within a legal hold policy by item type., defaults to None
                :type assignment_counts: Optional[LegalHoldPolicyAssignmentCountsField], optional
                :param created_at: When the legal hold policy object was created., defaults to None
                :type created_at: Optional[DateTime], optional
                :param modified_at: When the legal hold policy object was modified.
        Does not update when assignments are added or removed., defaults to None
                :type modified_at: Optional[DateTime], optional
                :param deleted_at: When the policy release request was sent. (Because
        it can take time for a policy to fully delete, this
        isn't quite the same time that the policy is fully deleted).

        If `null`, the policy was not deleted., defaults to None
                :type deleted_at: Optional[DateTime], optional
                :param filter_started_at: User-specified, optional date filter applies to
        Custodian assignments only., defaults to None
                :type filter_started_at: Optional[DateTime], optional
                :param filter_ended_at: User-specified, optional date filter applies to
        Custodian assignments only., defaults to None
                :type filter_ended_at: Optional[DateTime], optional
                :param release_notes: Optional notes about why the policy was created., defaults to None
                :type release_notes: Optional[str], optional
                :param type: The value will always be `legal_hold_policy`., defaults to LegalHoldPolicyMiniTypeField.LEGAL_HOLD_POLICY
                :type type: LegalHoldPolicyMiniTypeField, optional
        """
        super().__init__(id=id, type=type, **kwargs)
        self.policy_name = policy_name
        self.description = description
        self.status = status
        self.assignment_counts = assignment_counts
        self.created_by = created_by
        self.created_at = created_at
        self.modified_at = modified_at
        self.deleted_at = deleted_at
        self.filter_started_at = filter_started_at
        self.filter_ended_at = filter_ended_at
        self.release_notes = release_notes



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from typing import List

from box_sdk_gen.schemas.workflow_mini import WorkflowMiniTypeField

from box_sdk_gen.schemas.workflow_mini import WorkflowMini

from box_sdk_gen.schemas.user_base import UserBase

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class WorkflowFlowsTypeField(str, Enum):
    FLOW = 'flow'


class WorkflowFlowsTriggerTypeField(str, Enum):
    TRIGGER = 'trigger'


class WorkflowFlowsTriggerTriggerTypeField(str, Enum):
    WORKFLOW_MANUAL_START = 'WORKFLOW_MANUAL_START'


class WorkflowFlowsTriggerScopeTypeField(str, Enum):
    TRIGGER_SCOPE = 'trigger_scope'


class WorkflowFlowsTriggerScopeObjectTypeField(str, Enum):
    FOLDER = 'folder'


class WorkflowFlowsTriggerScopeObjectField(BaseObject):
    _discriminator = 'type', {'folder'}

    def __init__(
        self,
        *,
        type: Optional[WorkflowFlowsTriggerScopeObjectTypeField] = None,
        id: Optional[str] = None,
        **kwargs
    ):
        """
        :param type: The type of the object., defaults to None
        :type type: Optional[WorkflowFlowsTriggerScopeObjectTypeField], optional
        :param id: The id of the object., defaults to None
        :type id: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.type = type
        self.id = id


class WorkflowFlowsTriggerScopeField(BaseObject):
    _discriminator = 'type', {'trigger_scope'}

    def __init__(
        self,
        *,
        type: Optional[WorkflowFlowsTriggerScopeTypeField] = None,
        ref: Optional[str] = None,
        object: Optional[WorkflowFlowsTriggerScopeObjectField] = None,
        **kwargs
    ):
        """
        :param type: The trigger scope's resource type., defaults to None
        :type type: Optional[WorkflowFlowsTriggerScopeTypeField], optional
        :param ref: Indicates the path of the condition value to check., defaults to None
        :type ref: Optional[str], optional
        :param object: The object the `ref` points to., defaults to None
        :type object: Optional[WorkflowFlowsTriggerScopeObjectField], optional
        """
        super().__init__(**kwargs)
        self.type = type
        self.ref = ref
        self.object = object


class WorkflowFlowsTriggerField(BaseObject):
    _discriminator = 'type', {'trigger'}

    def __init__(
        self,
        *,
        type: Optional[WorkflowFlowsTriggerTypeField] = None,
        trigger_type: Optional[WorkflowFlowsTriggerTriggerTypeField] = None,
        scope: Optional[List[WorkflowFlowsTriggerScopeField]] = None,
        **kwargs
    ):
        """
        :param type: The trigger's resource type., defaults to None
        :type type: Optional[WorkflowFlowsTriggerTypeField], optional
        :param trigger_type: The type of trigger selected for this flow., defaults to None
        :type trigger_type: Optional[WorkflowFlowsTriggerTriggerTypeField], optional
        :param scope: List of trigger scopes., defaults to None
        :type scope: Optional[List[WorkflowFlowsTriggerScopeField]], optional
        """
        super().__init__(**kwargs)
        self.type = type
        self.trigger_type = trigger_type
        self.scope = scope


class WorkflowFlowsOutcomesTypeField(str, Enum):
    OUTCOME = 'outcome'


class WorkflowFlowsOutcomesActionTypeField(str, Enum):
    ADD_METADATA = 'add_metadata'
    ASSIGN_TASK = 'assign_task'
    COPY_FILE = 'copy_file'
    COPY_FOLDER = 'copy_folder'
    CREATE_FOLDER = 'create_folder'
    DELETE_FILE = 'delete_file'
    DELETE_FOLDER = 'delete_folder'
    LOCK_FILE = 'lock_file'
    MOVE_FILE = 'move_file'
    MOVE_FOLDER = 'move_folder'
    REMOVE_WATERMARK_FILE = 'remove_watermark_file'
    RENAME_FOLDER = 'rename_folder'
    RESTORE_FOLDER = 'restore_folder'
    SHARE_FILE = 'share_file'
    SHARE_FOLDER = 'share_folder'
    UNLOCK_FILE = 'unlock_file'
    UPLOAD_FILE = 'upload_file'
    WAIT_FOR_TASK = 'wait_for_task'
    WATERMARK_FILE = 'watermark_file'
    GO_BACK_TO_STEP = 'go_back_to_step'
    APPLY_FILE_CLASSIFICATION = 'apply_file_classification'
    APPLY_FOLDER_CLASSIFICATION = 'apply_folder_classification'
    SEND_NOTIFICATION = 'send_notification'


class WorkflowFlowsOutcomesIfRejectedTypeField(str, Enum):
    OUTCOME = 'outcome'


class WorkflowFlowsOutcomesIfRejectedActionTypeField(str, Enum):
    ADD_METADATA = 'add_metadata'
    ASSIGN_TASK = 'assign_task'
    COPY_FILE = 'copy_file'
    COPY_FOLDER = 'copy_folder'
    CREATE_FOLDER = 'create_folder'
    DELETE_FILE = 'delete_file'
    DELETE_FOLDER = 'delete_folder'
    LOCK_FILE = 'lock_file'
    MOVE_FILE = 'move_file'
    MOVE_FOLDER = 'move_folder'
    REMOVE_WATERMARK_FILE = 'remove_watermark_file'
    RENAME_FOLDER = 'rename_folder'
    RESTORE_FOLDER = 'restore_folder'
    SHARE_FILE = 'share_file'
    SHARE_FOLDER = 'share_folder'
    UNLOCK_FILE = 'unlock_file'
    UPLOAD_FILE = 'upload_file'
    WAIT_FOR_TASK = 'wait_for_task'
    WATERMARK_FILE = 'watermark_file'
    GO_BACK_TO_STEP = 'go_back_to_step'
    APPLY_FILE_CLASSIFICATION = 'apply_file_classification'
    APPLY_FOLDER_CLASSIFICATION = 'apply_folder_classification'
    SEND_NOTIFICATION = 'send_notification'


class WorkflowFlowsOutcomesIfRejectedField(BaseObject):
    _discriminator = 'type', {'outcome'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[WorkflowFlowsOutcomesIfRejectedTypeField] = None,
        name: Optional[str] = None,
        action_type: Optional[WorkflowFlowsOutcomesIfRejectedActionTypeField] = None,
        **kwargs
    ):
        """
        :param id: The identifier of the outcome., defaults to None
        :type id: Optional[str], optional
        :param type: The outcomes resource type., defaults to None
        :type type: Optional[WorkflowFlowsOutcomesIfRejectedTypeField], optional
        :param name: The name of the outcome., defaults to None
        :type name: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.name = name
        self.action_type = action_type


class WorkflowFlowsOutcomesField(BaseObject):
    _discriminator = 'type', {'outcome'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[WorkflowFlowsOutcomesTypeField] = None,
        name: Optional[str] = None,
        action_type: Optional[WorkflowFlowsOutcomesActionTypeField] = None,
        if_rejected: Optional[List[WorkflowFlowsOutcomesIfRejectedField]] = None,
        **kwargs
    ):
        """
                :param id: The identifier of the outcome., defaults to None
                :type id: Optional[str], optional
                :param type: The outcomes resource type., defaults to None
                :type type: Optional[WorkflowFlowsOutcomesTypeField], optional
                :param name: The name of the outcome., defaults to None
                :type name: Optional[str], optional
                :param if_rejected: If `action_type` is `assign_task` and the task is rejected, returns a
        list of outcomes to complete., defaults to None
                :type if_rejected: Optional[List[WorkflowFlowsOutcomesIfRejectedField]], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.name = name
        self.action_type = action_type
        self.if_rejected = if_rejected


class WorkflowFlowsField(BaseObject):
    _discriminator = 'type', {'flow'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[WorkflowFlowsTypeField] = None,
        trigger: Optional[WorkflowFlowsTriggerField] = None,
        outcomes: Optional[List[WorkflowFlowsOutcomesField]] = None,
        created_at: Optional[DateTime] = None,
        created_by: Optional[UserBase] = None,
        **kwargs
    ):
        """
        :param id: The identifier of the flow., defaults to None
        :type id: Optional[str], optional
        :param type: The flow's resource type., defaults to None
        :type type: Optional[WorkflowFlowsTypeField], optional
        :param created_at: When this flow was created., defaults to None
        :type created_at: Optional[DateTime], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type
        self.trigger = trigger
        self.outcomes = outcomes
        self.created_at = created_at
        self.created_by = created_by


class Workflow(WorkflowMini):
    def __init__(
        self,
        *,
        flows: Optional[List[WorkflowFlowsField]] = None,
        id: Optional[str] = None,
        type: Optional[WorkflowMiniTypeField] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        is_enabled: Optional[bool] = None,
        **kwargs
    ):
        """
        :param flows: A list of flows assigned to a workflow., defaults to None
        :type flows: Optional[List[WorkflowFlowsField]], optional
        :param id: The unique identifier for the workflow., defaults to None
        :type id: Optional[str], optional
        :param type: The value will always be `workflow`., defaults to None
        :type type: Optional[WorkflowMiniTypeField], optional
        :param name: The name of the workflow., defaults to None
        :type name: Optional[str], optional
        :param description: The description for a workflow., defaults to None
        :type description: Optional[str], optional
        :param is_enabled: Specifies if this workflow is enabled., defaults to None
        :type is_enabled: Optional[bool], optional
        """
        super().__init__(
            id=id,
            type=type,
            name=name,
            description=description,
            is_enabled=is_enabled,
            **kwargs
        )
        self.flows = flows



from enum import Enum

from box_sdk_gen.internal.base_object import BaseObject

from typing import Optional

from typing import Dict

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.serialization.json import deserialize

from box_sdk_gen.serialization.json import serialize

from box_sdk_gen.networking.fetch_options import ResponseFormat

from box_sdk_gen.schemas.task_assignments import TaskAssignments

from box_sdk_gen.schemas.client_error import ClientError

from box_sdk_gen.schemas.task_assignment import TaskAssignment

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.networking.auth import Authentication

from box_sdk_gen.networking.network import NetworkSession

from box_sdk_gen.networking.fetch_options import FetchOptions

from box_sdk_gen.networking.fetch_response import FetchResponse

from box_sdk_gen.internal.utils import prepare_params

from box_sdk_gen.internal.utils import to_string

from box_sdk_gen.internal.utils import ByteStream

from box_sdk_gen.serialization.json import sd_to_json

from box_sdk_gen.serialization.json import SerializedData


class CreateTaskAssignmentTaskTypeField(str, Enum):
    TASK = 'task'


class CreateTaskAssignmentTask(BaseObject):
    _discriminator = 'type', {'task'}

    def __init__(
        self,
        id: str,
        *,
        type: CreateTaskAssignmentTaskTypeField = CreateTaskAssignmentTaskTypeField.TASK,
        **kwargs
    ):
        """
        :param id: The ID of the task.
        :type id: str
        :param type: The type of the item to assign., defaults to CreateTaskAssignmentTaskTypeField.TASK
        :type type: CreateTaskAssignmentTaskTypeField, optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class CreateTaskAssignmentAssignTo(BaseObject):
    def __init__(
        self, *, id: Optional[str] = None, login: Optional[str] = None, **kwargs
    ):
        """
                :param id: The ID of the user to assign to the
        task.

        To specify a user by their email
        address use the `login` parameter., defaults to None
                :type id: Optional[str], optional
                :param login: The email address of the user to assign to the task.
        To specify a user by their user ID please use the `id` parameter., defaults to None
                :type login: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.login = login


class UpdateTaskAssignmentByIdResolutionState(str, Enum):
    COMPLETED = 'completed'
    INCOMPLETE = 'incomplete'
    APPROVED = 'approved'
    REJECTED = 'rejected'


class TaskAssignmentsManager:
    def __init__(
        self,
        *,
        auth: Optional[Authentication] = None,
        network_session: NetworkSession = None
    ):
        if network_session is None:
            network_session = NetworkSession()
        self.auth = auth
        self.network_session = network_session

    def get_task_assignments(
        self, task_id: str, *, extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> TaskAssignments:
        """
                Lists all of the assignments for a given task.
                :param task_id: The ID of the task.
        Example: "12345"
                :type task_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/tasks/',
                        to_string(task_id),
                        '/assignments',
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, TaskAssignments)

    def create_task_assignment(
        self,
        task: CreateTaskAssignmentTask,
        assign_to: CreateTaskAssignmentAssignTo,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> TaskAssignment:
        """
        Assigns a task to a user.

        A task can be assigned to more than one user by creating multiple


        assignments.

        :param task: The task to assign to a user.
        :type task: CreateTaskAssignmentTask
        :param assign_to: The user to assign the task to.
        :type assign_to: CreateTaskAssignmentAssignTo
        :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
        :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'task': task, 'assign_to': assign_to}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [self.network_session.base_urls.base_url, '/2.0/task_assignments']
                ),
                method='POST',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, TaskAssignment)

    def get_task_assignment_by_id(
        self,
        task_assignment_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> TaskAssignment:
        """
                Retrieves information about a task assignment.
                :param task_assignment_id: The ID of the task assignment.
        Example: "12345"
                :type task_assignment_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/task_assignments/',
                        to_string(task_assignment_id),
                    ]
                ),
                method='GET',
                headers=headers_map,
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, TaskAssignment)

    def update_task_assignment_by_id(
        self,
        task_assignment_id: str,
        *,
        message: Optional[str] = None,
        resolution_state: Optional[UpdateTaskAssignmentByIdResolutionState] = None,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> TaskAssignment:
        """
                Updates a task assignment. This endpoint can be

                used to update the state of a task assigned to a user.

                :param task_assignment_id: The ID of the task assignment.
        Example: "12345"
                :type task_assignment_id: str
                :param message: An optional message by the assignee that can be added to the task., defaults to None
                :type message: Optional[str], optional
                :param resolution_state: The state of the task assigned to the user.

        * For a task with an `action` value of `complete` this can be
        `incomplete` or `completed`.
        * For a task with an `action` of `review` this can be
        `incomplete`, `approved`, or `rejected`., defaults to None
                :type resolution_state: Optional[UpdateTaskAssignmentByIdResolutionState], optional
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        request_body: Dict = {'message': message, 'resolution_state': resolution_state}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/task_assignments/',
                        to_string(task_assignment_id),
                    ]
                ),
                method='PUT',
                headers=headers_map,
                data=serialize(request_body),
                content_type='application/json',
                response_format=ResponseFormat.JSON,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return deserialize(response.data, TaskAssignment)

    def delete_task_assignment_by_id(
        self,
        task_assignment_id: str,
        *,
        extra_headers: Optional[Dict[str, Optional[str]]] = None
    ) -> None:
        """
                Deletes a specific task assignment.
                :param task_assignment_id: The ID of the task assignment.
        Example: "12345"
                :type task_assignment_id: str
                :param extra_headers: Extra headers that will be included in the HTTP request., defaults to None
                :type extra_headers: Optional[Dict[str, Optional[str]]], optional
        """
        if extra_headers is None:
            extra_headers = {}
        headers_map: Dict[str, str] = prepare_params({**extra_headers})
        response: FetchResponse = self.network_session.network_client.fetch(
            FetchOptions(
                url=''.join(
                    [
                        self.network_session.base_urls.base_url,
                        '/2.0/task_assignments/',
                        to_string(task_assignment_id),
                    ]
                ),
                method='DELETE',
                headers=headers_map,
                response_format=ResponseFormat.NO_CONTENT,
                auth=self.auth,
                network_session=self.network_session,
            )
        )
        return None



from enum import Enum

from typing import Optional

from typing import Dict

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.metadata_field_filter_float_range import (
    MetadataFieldFilterFloatRange,
)

from box_sdk_gen.schemas.metadata_field_filter_date_range import (
    MetadataFieldFilterDateRange,
)

from box_sdk_gen.schemas.metadata_filter_value import MetadataFilterValue

from box_sdk_gen.box.errors import BoxSDKError


class MetadataFilterScopeField(str, Enum):
    GLOBAL = 'global'
    ENTERPRISE = 'enterprise'
    ENTERPRISE__ENTERPRISE_ID_ = 'enterprise_{enterprise_id}'


class MetadataFilter(BaseObject):
    _fields_to_json_mapping: Dict[str, str] = {
        'template_key': 'templateKey',
        **BaseObject._fields_to_json_mapping,
    }
    _json_to_fields_mapping: Dict[str, str] = {
        'templateKey': 'template_key',
        **BaseObject._json_to_fields_mapping,
    }

    def __init__(
        self,
        *,
        scope: Optional[MetadataFilterScopeField] = None,
        template_key: Optional[str] = None,
        filters: Optional[Dict[str, MetadataFilterValue]] = None,
        **kwargs
    ):
        """
                :param scope: Specifies the scope of the template to filter search results by.

        This will be `enterprise_{enterprise_id}` for templates defined
        for use in this enterprise, and `global` for general templates
        that are available to all enterprises using Box., defaults to None
                :type scope: Optional[MetadataFilterScopeField], optional
                :param template_key: The key of the template used to filter search results.

        In many cases the template key is automatically derived
        of its display name, for example `Contract Template` would
        become `contractTemplate`. In some cases the creator of the
        template will have provided its own template key.

        Please [list the templates for an enterprise][list], or
        get all instances on a [file][file] or [folder][folder]
        to inspect a template's key.

        [list]: https://developer.box.com/reference/get-metadata-templates-enterprise
        [file]: https://developer.box.com/reference/get-files-id-metadata
        [folder]: https://developer.box.com/reference/get-folders-id-metadata, defaults to None
                :type template_key: Optional[str], optional
                :param filters: Specifies which fields on the template to filter the search
        results by. When more than one field is specified, the query
        performs a logical `AND` to ensure that the instance of the
        template matches each of the fields specified., defaults to None
                :type filters: Optional[Dict[str, MetadataFilterValue]], optional
        """
        super().__init__(**kwargs)
        self.scope = scope
        self.template_key = template_key
        self.filters = filters



from typing import Union

from box_sdk_gen.schemas.ai_agent_ask import AiAgentAsk

from box_sdk_gen.schemas.ai_agent_text_gen import AiAgentTextGen

from box_sdk_gen.schemas.ai_agent_extract import AiAgentExtract

from box_sdk_gen.schemas.ai_agent_extract_structured import AiAgentExtractStructured

from box_sdk_gen.box.errors import BoxSDKError

AiAgent = Union[AiAgentAsk, AiAgentTextGen, AiAgentExtract, AiAgentExtractStructured]



from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class UploadUrl(BaseObject):
    def __init__(
        self,
        *,
        upload_url: Optional[str] = None,
        upload_token: Optional[str] = None,
        **kwargs
    ):
        """
                :param upload_url: A URL for an upload session that can be used to upload
        the file., defaults to None
                :type upload_url: Optional[str], optional
                :param upload_token: An optional access token to use to upload the file., defaults to None
                :type upload_token: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.upload_url = upload_url
        self.upload_token = upload_token



from enum import Enum

from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.schemas.shield_information_barrier_segment_member_base import (
    ShieldInformationBarrierSegmentMemberBaseTypeField,
)

from box_sdk_gen.schemas.shield_information_barrier_segment_member_base import (
    ShieldInformationBarrierSegmentMemberBase,
)

from box_sdk_gen.schemas.shield_information_barrier_segment_member_mini import (
    ShieldInformationBarrierSegmentMemberMini,
)

from box_sdk_gen.schemas.shield_information_barrier_base import (
    ShieldInformationBarrierBase,
)

from box_sdk_gen.schemas.user_base import UserBase

from box_sdk_gen.box.errors import BoxSDKError

from box_sdk_gen.internal.utils import DateTime


class ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField(
    str, Enum
):
    SHIELD_INFORMATION_BARRIER_SEGMENT = 'shield_information_barrier_segment'


class ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentField(
    BaseObject
):
    _discriminator = 'type', {'shield_information_barrier_segment'}

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        type: Optional[
            ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField
        ] = None,
        **kwargs
    ):
        """
                :param id: The ID reference of the requesting
        shield information barrier segment., defaults to None
                :type id: Optional[str], optional
                :param type: The type of the shield information barrier segment., defaults to None
                :type type: Optional[ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentTypeField], optional
        """
        super().__init__(**kwargs)
        self.id = id
        self.type = type


class ShieldInformationBarrierSegmentMember(ShieldInformationBarrierSegmentMemberMini):
    def __init__(
        self,
        *,
        shield_information_barrier: Optional[ShieldInformationBarrierBase] = None,
        shield_information_barrier_segment: Optional[
            ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentField
        ] = None,
        created_at: Optional[DateTime] = None,
        created_by: Optional[UserBase] = None,
        updated_at: Optional[DateTime] = None,
        updated_by: Optional[UserBase] = None,
        user: Optional[UserBase] = None,
        id: Optional[str] = None,
        type: Optional[ShieldInformationBarrierSegmentMemberBaseTypeField] = None,
        **kwargs
    ):
        """
                :param shield_information_barrier_segment: The `type` and `id` of the requested
        shield information barrier segment., defaults to None
                :type shield_information_barrier_segment: Optional[ShieldInformationBarrierSegmentMemberShieldInformationBarrierSegmentField], optional
                :param created_at: ISO date time string when this shield
        information barrier object was created., defaults to None
                :type created_at: Optional[DateTime], optional
                :param updated_at: ISO date time string when this
        shield information barrier segment Member was updated., defaults to None
                :type updated_at: Optional[DateTime], optional
                :param id: The unique identifier for the
        shield information barrier segment member., defaults to None
                :type id: Optional[str], optional
                :param type: The type of the shield information barrier segment member., defaults to None
                :type type: Optional[ShieldInformationBarrierSegmentMemberBaseTypeField], optional
        """
        super().__init__(user=user, id=id, type=type, **kwargs)
        self.shield_information_barrier = shield_information_barrier
        self.shield_information_barrier_segment = shield_information_barrier_segment
        self.created_at = created_at
        self.created_by = created_by
        self.updated_at = updated_at
        self.updated_by = updated_by



from typing import Optional

from box_sdk_gen.internal.base_object import BaseObject

from box_sdk_gen.box.errors import BoxSDKError


class HubCollaborationUpdateRequestV2025R0(BaseObject):
    def __init__(self, *, role: Optional[str] = None, **kwargs):
        """
                :param role: The level of access granted to a Box Hub.
        Possible values are `editor`, `viewer`, and `co-owner`., defaults to None
                :type role: Optional[str], optional
        """
        super().__init__(**kwargs)
        self.role = role
